// -*- C++ -*-
//
// user_ana.cc
// Template source file
//
// Author:  
// Created: 
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include "E391Detector/E391Detector.h"
#include "TupleManager/TupleManager.h"
#include "TupleManager/TMNtuple.h"

extern TupleManager  otuple;
extern const E391Detector&  e391det;


// User defined Ntuples
TMNtuple*  ntEvent;
TMNtuple*  ntScaler;


// init routine
void
user_init()
{
  // Histogram definition
  ntEvent = otuple.ntupleR( 1,"Event Data",
                            "runno spill evtno trig "
                            "adc0 adc1 adc2 adc3 adc8 adc9 "
                            "tdc0 tdc1 tdc2 tdc3 tdc8 tdc9 "
                            );

  ntScaler = otuple.ntupleR( 2,"Scaler Data",
                             "runno spill "
                             "scl0  scl1  scl2  scl3  scl4  scl5 "
                             "dscl0 dscl1 dscl2 dscl3 dscl4 dscl5 "
                             );

}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  // 
}


// process_event routine
void
user_process_event( int run_number, int spill_number, int event_number )
{
  static int  ct = 0;
  if ( ++ct % 100 == 0 ){
    std::cout << "count=" << ct << " Run" << run_number 
              << " spill=" << spill_number << " Event " << event_number
              << std::endl;
  }

  
  if ( event_number > 0 ){ 
    // Event Data
    if ( e391det.adcCrate(1).valid() ){
      int  trig = e391det.adcCrate(1).trg() & 0xffff;
      
      ntEvent->fill( "runno", run_number );
      ntEvent->fill( "spill", spill_number );
      ntEvent->fill( "evtno", event_number );
      ntEvent->fill( "trig",  trig );
      ntEvent->fill( "adc0",  e391det.CsI(0).adc() );
      ntEvent->fill( "adc1",  e391det.CsI(1).adc() );
      ntEvent->fill( "adc2",  e391det.CsI(2).adc() );
      ntEvent->fill( "adc3",  e391det.CsI(3).adc() );
      ntEvent->fill( "adc8",  e391det.CsI(8).adc() );
      ntEvent->fill( "adc9",  e391det.CsI(9).adc() );
      ntEvent->fill( "tdc0",  e391det.CsI(0).tdc() );
      ntEvent->fill( "tdc1",  e391det.CsI(1).tdc() );
      ntEvent->fill( "tdc2",  e391det.CsI(2).tdc() );
      ntEvent->fill( "tdc3",  e391det.CsI(3).tdc() );
      ntEvent->fill( "tdc8",  e391det.CsI(8).tdc() );
      ntEvent->fill( "tdc9",  e391det.CsI(9).tdc() );
      
      ntEvent->dump();
      ntEvent->clear();
    }
  }

  else {
    // Scaler Data
    int*  pscaler = (int*)e391det.pbank( "SCL0" );
    if ( pscaler ){
      static bool  init = false;
      static int   oldrun = -1;
      static const int  Nscl = 6;
      static int  initscl[Nscl];
      static int  oldscl[Nscl];
      static char* c_scl[Nscl] = {
        "scl0","scl1","scl2","scl3","scl4","scl5"
      };
      static char* c_dscl[Nscl] = { 
        "dscl0","dscl1","dscl2","dscl3","dscl4","dscl5"
      };
      if ( oldrun != run_number ){
        oldrun = run_number;
        init = false;
      }
      if ( !init ){ // initial value
        for( int i=0;i<Nscl;i++ ){
          initscl[i] = oldscl[i] = pscaler[i];
        }
        init = true;
      }
      else{
        ntScaler->fill( "runno", run_number );
        ntScaler->fill( "spill", spill_number );
        for( int i=0;i<Nscl;i++ ){
          ntScaler->fill( c_scl[i], pscaler[i]-initscl[i] );
          ntScaler->fill( c_dscl[i], pscaler[i]-oldscl[i]  );
          oldscl[i] = pscaler[i];
        }
        ntScaler->dump();
        ntScaler->clear();
      }
    }
  }

}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
}


// term routine
void
user_term()
{
  //
}


