// -*- C++ -*-
//
// eventbuilder.cc
// 
// Mon Jul  8 22:27:07 JST 2002
// M.Yamaga
//
// $Id$
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <list>
#include <unistd.h>
#include <stdio.h>
#include "midas.h"
#include "unpack/TdcModule.h"
#include "unpack/TdcEvent.h"
#include "unpack/AdcModule.h"
#include "unpack/AdcEvent.h"
#include "MidasManager/MidasBank.h"
#include "MidasManager/MidasFileIO.h"


#define  DEBUG  0

//
INT event_builder( EVENT_HEADER*, void* );
INT event_builder_init( void );
INT event_builder_bor( INT run_number );
INT event_builder_eor( INT run_number );

//
ANA_MODULE event_builder_module = {
  "Event Builder",               /* module name           */  
  "Mitsuhiro YAMAGA",            /* author                */
  event_builder,                 /* event routine         */
  event_builder_bor,             /* BOR routine           */
  event_builder_eor,             /* EOR routine           */
  event_builder_init,            /* init routine          */
  NULL,                          /* exit routine          */
  NULL,                          /* parameter structure   */
  0,                             /* structure size        */
  NULL,                          /* initial parameters    */
};


//
extern RUNINFO  runinfo; // analyzer.cc

extern FILE*    out_file; // mana.cc
extern bool     out_gzip; // mana.cc

//
static MidasBank  mbank;
static MidasFileIO  mfile;

static std::list<TdcEvent*>  tdc0List;
static std::list<TdcEvent*>  tdc1List;
static std::list<AdcEvent*>  adc0List;
static std::list<AdcEvent*>  adc1List;

static int  event_counter = 0;


////// dumpEvent
static void
dumpTriggerEvent( const EVENT_HEADER* pheader, 
                  const AdcEvent* padc0,
                  const AdcEvent* padc1,
                  const TdcEvent* ptdc0,
                  const TdcEvent* ptdc1 )
{
#if DEBUG
  std::cout << "dumpTriggerEvent : pheader=" << pheader << std::endl;
#endif

  //
  mbank.init( pheader->event_id,pheader->trigger_mask,
              pheader->serial_number );
  
  EVENT_HEADER*  phd = mbank.eventHeader();
  void*          pev = mbank.eventBuffer();

  phd->time_stamp = pheader->time_stamp;

  event_counter++;

#if DEBUG
  std::cout << " Event" << event_counter << " : Saved Bank :";
#endif

  // check validity of each bank
  unsigned int  valid = 0;
  if ( padc1 && padc1->valid() )
    valid |= (0x01 << 3);
  if ( padc0 && padc0->valid() )
    valid |= (0x01 << 2);
  if ( ptdc1 && ptdc1->valid() )
    valid |= (0x01 << 1);
  if ( ptdc0 && ptdc0->valid() )
    valid |= (0x01 << 0);

  // start
  DWORD*  pdata;

  // EVNT
  bk_create( pev, "EVNT", TID_DWORD, &pdata );
  *pdata++ = 1;                      // EXP=1, MC=2
  *pdata++ = runinfo.run_number;     // Run Number
  *pdata++ = pheader->serial_number; // Serial (spill) Number
  *pdata++ = event_counter;          // Event Number
  *pdata++ = valid;                  // Validity
  bk_close( pev, pdata );

  // ADC0, NGF,TRG
  if ( padc0 ){
    bk_create( pev, "NGF0", TID_DWORD, &pdata );
    for( std::list<unsigned int>::const_iterator i= padc0->ngf().begin();
         i != padc0->ngf().end(); i++ ){
      *pdata++ = *i;
    }
    bk_close(pev, pdata);
    //
    bk_create( pev, "TRG0", TID_DWORD, &pdata );
    *pdata++ = padc0->trg();
    bk_close(pev, pdata);
    //
    bk_create( pev, "ADC0", TID_DWORD, &pdata );
    *pdata++ = (0x01 << 16) | (padc0->nhit() + 1);
    for( std::list<AdcModule>::const_iterator m=padc0->adcList().begin();
         m!=padc0->adcList().end();m++ ){
      for( std::list<unsigned int>::const_iterator h=m->hitList().begin();
           h!=m->hitList().end(); h++ ){
        *pdata++ = *h;
      }
    }
    *pdata++ = 0xff000000;
    *pdata++ = 0xffffffff;
    bk_close(pev, pdata);
#if DEBUG
    std::cout << " ADC0";
#endif
  }

  // ADC1
  if ( padc1 ){
    bk_create( pev, "NGF1", TID_DWORD, &pdata );
    for( std::list<unsigned int>::const_iterator i= padc1->ngf().begin();
         i != padc1->ngf().end(); i++ ){
      *pdata++ = *i;
    }
    bk_close(pev, pdata);
    //
    bk_create( pev, "TRG1", TID_DWORD, &pdata );
    *pdata++ = padc1->trg();
    bk_close(pev, pdata);
    //
    bk_create( pev, "ADC1", TID_DWORD, &pdata );
    *pdata++ = (0x01 << 16) | (padc1->nhit() + 1);
    for( std::list<AdcModule>::const_iterator m=padc1->adcList().begin();
         m!=padc1->adcList().end();m++ ){
      for( std::list<unsigned int>::const_iterator h=m->hitList().begin();
           h!=m->hitList().end(); h++ ){
        *pdata++ = *h;
      }
    }
    *pdata++ = 0xff000000;
    *pdata++ = 0xffffffff;
    bk_close(pev, pdata);
#if DEBUG
    std::cout << " ADC1";
#endif
  }

  // TDC0, SMP0
  if ( ptdc0 ){
    bk_create( pev, "SMP0", TID_DWORD, &pdata );
    for( std::list<unsigned int>::const_iterator m=ptdc0->smp().begin();
         m!=ptdc0->smp().end(); m++ ){
      *pdata++ = *m;
    }
    bk_close(pev, pdata);
    //
    bk_create( pev, "TDC0", TID_DWORD, &pdata );
    *pdata++ = (0x01 << 16) | (ptdc0->nhit() + 1);
    for( std::list<TdcModule>::const_iterator m=ptdc0->tdcList().begin();
         m!=ptdc0->tdcList().end();m++ ){
      for( std::list<unsigned int>::const_iterator h=m->hitList().begin();
           h!=m->hitList().end(); h++ ){
        *pdata++ = *h;
      }
    }
    *pdata++ = ptdc0->flag();
    *pdata++ = 0xffffffff;
    bk_close(pev, pdata);
#if DEBUG
    std::cout << " TDC0";
#endif
  }

  // TDC1, SMP1
  if ( ptdc1 ){
    bk_create( pev, "SMP1", TID_DWORD, &pdata );
    for( std::list<unsigned int>::const_iterator m=ptdc1->smp().begin();
         m!=ptdc1->smp().end(); m++ ){
      *pdata++ = *m;
    }
    bk_close(pev, pdata);
    //
    bk_create( pev, "TDC1", TID_DWORD, &pdata );
    *pdata++ = (0x01 << 16) | (ptdc1->nhit() + 1);
    for( std::list<TdcModule>::const_iterator m=ptdc1->tdcList().begin();
         m!=ptdc1->tdcList().end();m++ ){
      for( std::list<unsigned int>::const_iterator h=m->hitList().begin();
           h!=m->hitList().end(); h++ ){
        *pdata++ = *h;
      }
    }
    *pdata++ = ptdc1->flag();
    *pdata++ = 0xffffffff;
    bk_close(pev, pdata);
#if DEBUG
    std::cout << " TDC1";
#endif
  }

#if DEBUG
  std::cout << std::endl;
#endif
  //
  if ( out_file ){
    mfile.writeEvent();
  }

}


//
static void
dumpScalerEvent( EVENT_HEADER* pheader, void* pevent )
{
#if DEBUG
  std::cout << "dumpScalerEvent" << std::endl;
#endif

  mbank.init( pheader->event_id, pheader->trigger_mask,
              pheader->serial_number );

  EVENT_HEADER*  phd = mbank.eventHeader();
  void*          pev = mbank.eventBuffer();

  phd->time_stamp = pheader->time_stamp;

  // Scaler event
  DWORD*  pdataSCL;
  int  sclsize = bk_locate( pevent, "SCL0", &pdataSCL );
  
  DWORD*  pdata;
  bk_create( pev, "SCL0", TID_DWORD, &pdata );
  for( int i=0;i<sclsize;i++ ){
    *pdata++ = pdataSCL[i];
  }      
  bk_close(pev, pdata);
  
#if DEBUG
  std::cout << "SCL0 Bank saved." << std::endl;
#endif

  if ( out_file ){
    mfile.writeEvent();
  }

}

  
//
static bool
processEvent()
{
#if DEBUG
  std::cout << "processEvent" << std::endl;
#endif

  // check EventNumber consistency

  // obtain eventNumber
  // TDC0
  int  tdc0evnum = -1;
  bool tdc0valid = false;
  if ( !tdc0List.empty() && (*tdc0List.begin())->valid() ){
    tdc0evnum = (*tdc0List.begin())->tdc( 16 ).hitPattern() >> 56;
    tdc0evnum &= 0x7f;
    tdc0valid = true;
    //tdc0evnum = (tdc0evnum+10) & 0x7f; // for test
  }

  // TDC1
  int  tdc1evnum = -1;
  bool tdc1valid = false;
  if ( !tdc1List.empty() && (*tdc1List.begin())->valid() ){
    tdc1evnum = (*tdc1List.begin())->tdc( 15 ).hitPattern() >> 56;
    tdc1evnum &= 0x7f;
    tdc1valid = true;
    //tdc1evnum = (tdc1evnum+10) & 0x7f; // for test
  }
    
  // ADC0
  int  adc0evnum = -1;
  bool adc0valid = false;
  if ( !adc0List.empty() && (*adc0List.begin())->valid() ){
    adc0evnum = (*adc0List.begin())->adc( 18 ).hitPatternH(1000) >> 40;
    adc0evnum &= 0x7f;
    //adc0evnum = (adc0evnum+2) & 0x7f; // for test
    //
    if ( (*adc0List.begin())->ngf(2) & 0x0080 ){ // 0x0080 = FIFO full
      (*adc0List.begin())->setValid( false ); // invalidate this event
#if 1|DEBUG
      std::cout << "processEvent : invalidate ADC0 : NGF fifo=" 
                << std::hex << std::setw(8) << std::setfill('0')
                << (*adc0List.begin())->ngf(2)
                << std::dec << std::endl;
#endif
    }
    else{
      adc0valid = true;
    }
  }

  // ADC1
  int  adc1evnum = -1;
  bool adc1valid = false;
  if ( !adc1List.empty() && (*adc1List.begin())->valid() ){
    adc1evnum = (*adc1List.begin())->adc( 18 ).hitPatternH(1000) >> 40;
    adc1evnum &= 0x7f;
    //adc1evnum = (adc0evnum+5) & 0x7f; // for test
    //
    if ( (*adc1List.begin())->ngf(2) & 0x0080 ){ // 0x0080 = FIFO full
      (*adc1List.begin())->setValid( false ); // invalidate this event
#if 1|DEBUG
      std::cout << "processEvent : invalidate ADC1 : NGF fifo=" 
                << std::hex << std::setw(8) << std::setfill('0')
                << (*adc1List.begin())->ngf(2)
                << std::dec << std::endl;
#endif
    }
    else{
      adc1valid = true;
    }
  }

#if 1
  if ( !(tdc0evnum == tdc1evnum && 
         tdc0evnum == adc0evnum &&
         tdc0evnum == adc1evnum) ){
    std::cout << " EventNumber : serial=" << event_counter
              << " : TDC(0:16:56)="
              << std::hex << std::setw(2) << std::setfill('0')
              << tdc0evnum
              << " TDC(1:15:56)="
              << std::hex << std::setw(2) << std::setfill('0')
              << tdc1evnum
              << " ADC(0:18:88)="
              << std::hex << std::setw(2) << std::setfill('0')
              << adc0evnum
              << " ADC(1:18:88)="
              << std::hex << std::setw(2) << std::setfill('0')
              << adc1evnum
              << std::dec
              << " ## SLIP ##" << std::endl;
    //if ( tdc0evnum == adc0evnum && adc0evnum == adc1evnum ){
    //  std::cout << "   OK" << std::endl;
    //}
    //else{
    //std::cout << " ## SLIP ##" << std::endl;
  }
#endif

#if 0 // debug dump of NGF status
  if ( adc0evnum == 0 ){
    std::cout << "ADC0 : evnum=0x00 : NFG=";
    for( std::list<unsigned int>::const_iterator 
           i = (*adc0List.begin())->ngf().begin();
         i != (*adc0List.begin())->ngf().end();
         i++ ){
      std::cout << std::hex << std::setw(8) << std::setfill('0')
                << *i << std::dec << ",";
    }
    std::cout << std::endl;
  }
#endif
    

  // find minimum eventNumber
  int  min = -1;
  if ( tdc0evnum >= 0 ){
    min = tdc0evnum;
  }
  //
  if ( tdc1evnum >= 0 ){
    if ( min >= 0 ){
      if ( abs(min - tdc1evnum) < 0x40 ){
        if ( tdc1evnum < min )
          min = tdc1evnum;
      }
      else{
        if ( tdc1evnum > min )
          min = tdc1evnum;
      }
    }
    else{
      min = tdc1evnum;
    }
  }
  //
  if ( adc0evnum >= 0 ){
    if ( min >= 0 ){
      if ( abs(min - adc0evnum) < 0x40 ){
        if ( adc0evnum < min )
          min = adc0evnum;
      }
      else{
        if ( adc0evnum > min )
          min = adc0evnum;
      }
    }
    else{
      min = adc0evnum;
    }
  }
  //
  if ( adc1evnum >= 0 ){
    if ( min >= 0 ){
      if ( abs(min - adc1evnum) < 0x40 ){
        if ( adc1evnum < min )
          min = adc1evnum;
      }
      else{
        if ( adc1evnum > min )
          min = adc1evnum;
      }
    }
    else{
      min = adc1evnum;
    }
  }
  // min validity
  if ( min < 0 )
    return( false );

  // find matched event
  AdcEvent*  padc0 = 0;
  AdcEvent*  padc1 = 0;
  TdcEvent*  ptdc0 = 0;
  TdcEvent*  ptdc1 = 0;
  const EVENT_HEADER*  pheader;
  if ( adc0evnum == min ){
    padc0 = *adc0List.begin();
    pheader = &padc0->eventHeader();
  }
  if ( adc1evnum == min ){
    padc1 = *adc1List.begin();
    pheader = &padc1->eventHeader();
  }
  if ( tdc0evnum == min ){
    ptdc0 = *tdc0List.begin();
    pheader = &ptdc0->eventHeader();
  }
  if ( tdc1evnum == min ){
    ptdc1 = *tdc1List.begin();
    pheader = &ptdc1->eventHeader();
  }
  // dump this event
  dumpTriggerEvent( pheader, padc0,padc1,ptdc0,ptdc1 );
  
  // remove from the list
  if ( ptdc0 != 0 ){
    delete  ptdc0;
    ptdc0 = 0;
    tdc0List.pop_front();
  }
  if ( ptdc1 != 0 ){
    delete  ptdc1;
    ptdc1 = 0;
    tdc1List.pop_front();
  }
  if ( padc0 != 0 ){
    delete  padc0;
    padc0 = 0;
    adc0List.pop_front();
  }
  if ( padc1 != 0 ){
    delete  padc1;
    padc1 = 0;
    adc1List.pop_front();
  }
  
#if DEBUG
  std::cout << "  ListSize :" 
            << " TDC0=" << tdc0List.size()
            << " TDC1=" << tdc1List.size()
            << " ADC0=" << adc0List.size()
            << " ADC1=" << adc1List.size()
            << std::endl;
#endif
  
  // find invalid data
  if ( !adc0valid && !adc0List.empty() ){
    padc0 = *adc0List.begin();
    pheader = &padc0->eventHeader();
  }
  if ( !adc1valid && !adc1List.empty() ){
    padc1 = *adc1List.begin();
    pheader = &padc1->eventHeader();
  }
  if ( !tdc0valid && !tdc0List.empty() ){
    ptdc0 = *tdc0List.begin();
    pheader = &ptdc0->eventHeader();
  }
  if ( !tdc1valid && !tdc1List.empty() ){
    ptdc1 = *tdc1List.begin();
    pheader = &ptdc1->eventHeader();
  }
  
  // dump invalid event
  if ( padc0 || padc1 || ptdc0 || ptdc1 ){
    dumpTriggerEvent( pheader, padc0,padc1,ptdc0,ptdc1 );
    
    // remove from the list
    if ( ptdc0 != 0 ){
      delete  ptdc0;
      ptdc0 = 0;
      tdc0List.pop_front();
    }
    if ( ptdc1 != 0 ){
      delete  ptdc1;
      ptdc1 = 0;
      tdc1List.pop_front();
    }
    if ( padc0 != 0 ){
      delete  padc0;
      padc0 = 0;
      adc0List.pop_front();
    }
    if ( padc1 != 0 ){
      delete  padc1;
      padc1 = 0;
      adc1List.pop_front();
    }
    
#if DEBUG
    std::cout << "  ListSize :" 
              << " TDC0=" << tdc0List.size()
              << " TDC1=" << tdc1List.size()
              << " ADC0=" << adc0List.size()
              << " ADC1=" << adc1List.size()
              << std::endl;
#endif
  }

  //
  return( true );
}


//////
INT
event_builder( EVENT_HEADER* pheader, void* pevent )
{
  std::cout << "EventBuilder:"
            << " Run#" << runinfo.run_number
            << " EventID#" << pheader->event_id
            << " Serial#" << pheader->serial_number
            << std::endl;

  // reset counter
  event_counter = 0;

  // Nevent in the spill
  DWORD*  pdataSMP0=0;
  int  ndataSMP0 = bk_locate( pevent, "SMP0", &pdataSMP0 );
  int  neventSMP0=0;
  bool  overflowSMP0 = false;
  if ( ndataSMP0 >= 3 && pdataSMP0 != 0 ){
    neventSMP0 = pdataSMP0[1] & 0xffff;
    if ( pdataSMP0[2] >> 31 )
      overflowSMP0 = true;
  }

  DWORD*  pdataSMP1=0;
  int  ndataSMP1 = bk_locate( pevent, "SMP1", &pdataSMP1 );
  int  neventSMP1=0;
  bool  overflowSMP1 = false;
  if ( ndataSMP1 >= 3 && pdataSMP1 != 0 ){
    neventSMP1 = pdataSMP1[1] & 0xffff;
    if ( pdataSMP1[2] >> 31 )
      overflowSMP1 = true;
  }

  DWORD*  pdataNGF0=0;
  int  ndataNGF0 = bk_locate( pevent, "NGF0", &pdataNGF0 );
  int  neventNGF0=0;
  bool  overflowNGF0 = false;
  if ( ndataNGF0 > 0 && pdataNGF0 != 0 ){
    neventNGF0 = pdataNGF0[0] & 0x0fffffff;
    if ( pdataNGF0[0] >> 31 )
      overflowNGF0 = true;
  }
  
  DWORD*  pdataNGF1=0;
  int  ndataNGF1 = bk_locate( pevent, "NGF1", &pdataNGF1 );
  int  neventNGF1=0;
  bool  overflowNGF1 = false;
  if ( ndataNGF1 > 0 && pdataNGF1 != 0 ){
    neventNGF1 = pdataNGF1[0] & 0x0fffffff;
    if ( pdataNGF1[0] >> 31 )
      overflowNGF1 = true;
  }

  // Trigger bank
  DWORD*  pdataTRG0=0;
  int  ndataTRG0 = bk_locate( pevent, "TRG0", &pdataTRG0 );

  DWORD*  pdataTRG1=0;
  int  ndataTRG1 = bk_locate( pevent, "TRG1", &pdataTRG1 );
  
  // dump
#if DEBUG
  std::cout << "  Nevent :"
            << " SMP0=" << neventSMP0 
            << " SMP1=" << neventSMP1 
            << " NGF0=" << neventNGF0 
            << " NGF1=" << neventNGF1;

  if ( neventSMP0 == neventSMP1 && 
       neventSMP0 == neventNGF0 && 
       neventSMP0 == neventNGF1  ){
    std::cout << "   OK" << std::endl;
  }
  else{
    std::cout << " ## SLIP ##" << std::endl;
  }
#endif

  // TDC0, TDC1, ADC0, ADC1 bank 
  DWORD*  pdataTDC0 = 0;
  int  tdc0size = bk_locate( pevent, "TDC0", &pdataTDC0 );
  DWORD*  pdataTDC1 = 0;
  int  tdc1size = bk_locate( pevent, "TDC1", &pdataTDC1 );
  DWORD*  pdataADC0 = 0;
  int  adc0size = bk_locate( pevent, "ADC0", &pdataADC0 );
  DWORD*  pdataADC1 = 0;
  int  adc1size = bk_locate( pevent, "ADC1", &pdataADC1 );

  // initialize serial#
  int  serialNumberTDC0 = 1;
  int  serialNumberTDC1 = 1;
  int  serialNumberADC0 = 1;
  int  serialNumberADC1 = 1;

  // loop over events in this spill
  while( true ){
    
    // check EventNumber consistency

    // obtain eventNumber
    // TDC0,SMP0
    if ( tdc0List.empty() ){
      // next event
      TdcEvent*  ptdc0 = new TdcEvent( 0, pheader, pdataTDC0, tdc0size,
                                       serialNumberTDC0 );
      if ( ptdc0->serial() == serialNumberTDC0 ){
        ptdc0->smp( pdataSMP0,ndataSMP0 ); // SMP
        tdc0List.push_back( ptdc0 );
        serialNumberTDC0++;
      }
      else{
        delete  ptdc0;
#if DEBUG
        std::cout << "No TDC0 event : serial#" << serialNumberTDC0 
                  << std::endl;
#endif
        if ( !overflowSMP0 ) // SMP0 overflow : Dump without SMP0
          break; // No more event : stop
      }
    }

    // TDC1,SMP1
    if ( tdc1List.empty() ){
      // next event
      TdcEvent*  ptdc1 = new TdcEvent( 1, pheader, pdataTDC1, tdc1size,
                                       serialNumberTDC1 );
      if ( ptdc1->serial() == serialNumberTDC1 ){
        ptdc1->smp( pdataSMP1,ndataSMP1 ); // SMP
        tdc1List.push_back( ptdc1 );
        serialNumberTDC1++;
      }
      else{
        delete  ptdc1;
#if DEBUG
        std::cout << "No TDC1 event : serial#" << serialNumberTDC1 
                  << std::endl;
#endif
        if ( !overflowSMP1 ) // SMP1 overflow : Dump without SMP1
          break; // No more event : stop
      }
    }

    // ADC0,NFG0,TRG0
    if ( adc0List.empty() ){
      // next event
      AdcEvent*  padc0 = new AdcEvent( 0, pheader, pdataADC0, adc0size,
                                       serialNumberADC0 );
      if ( padc0->serial() == serialNumberADC0 ){
        padc0->ngf( pdataNGF0,ndataNGF0 );
        padc0->trg( pdataTRG0,ndataTRG0 );
        adc0List.push_back( padc0 );
        serialNumberADC0++;
      }
      else{
        delete  padc0;
#if DEBUG
        std::cout << "No ADC0 event : serial#" << serialNumberADC0 
                  << std::endl;
#endif
        if ( !overflowNGF0 ) // NGF0 overflow : Dump without NGF0
          break; // No more event : stop
      }
    }
    
    // ADC1,NFG1,TRG1
    if ( adc1List.empty() ){
      // next event
      AdcEvent*  padc1 = new AdcEvent( 1, pheader, pdataADC1, adc1size,
                                       serialNumberADC1 );
      if ( padc1->serial() == serialNumberADC1 ){
        padc1->ngf( pdataNGF1,ndataNGF1 );
        padc1->trg( pdataTRG1,ndataTRG1 );
        adc1List.push_back( padc1 );
        serialNumberADC1++;
      }
      else{
        delete  padc1;
#if DEBUG
        std::cout << "No ADC1 event : serial#" << serialNumberADC1 
                  << std::endl;
#endif
        if ( !overflowNGF1 ) // NGF1 overflow : Dump without NGF1
          break; // No more event : stop
      }
    }

    // process this event
    if ( !processEvent() )
      break; // failed
  }

  // push_back remained events
  while( true ){
    // next event
    TdcEvent*  ptdc0 = new TdcEvent( 0, pheader, pdataTDC0, tdc0size,
                                     serialNumberTDC0 );
    if ( ptdc0->serial() == serialNumberTDC0 ){
      ptdc0->smp( pdataSMP0,ndataSMP0 ); // SMP
      tdc0List.push_back( ptdc0 );
      serialNumberTDC0++;
    }
    else{
      delete  ptdc0;
      break;
    }
  }

  while( true ){
    // next event
    TdcEvent*  ptdc1 = new TdcEvent( 1, pheader, pdataTDC1, tdc1size,
                                     serialNumberTDC1 );
    if ( ptdc1->serial() == serialNumberTDC1 ){
      ptdc1->smp( pdataSMP1,ndataSMP1 ); // SMP
      tdc1List.push_back( ptdc1 );
      serialNumberTDC1++;
    }
    else{
      delete  ptdc1;
      break;
    }
  }

  while( true ){
    // next event
    AdcEvent*  padc0 = new AdcEvent( 0, pheader, pdataADC0, adc0size,
                                     serialNumberADC0 );
    if ( padc0->serial() == serialNumberADC0 ){
      padc0->ngf( pdataNGF0,ndataNGF0 );
      padc0->trg( pdataTRG0,ndataTRG0 );
      adc0List.push_back( padc0 );
      serialNumberADC0++;
    }
    else{
      delete  padc0;
      break;
    }
  }

  while( true ){
    // next event
    AdcEvent*  padc1 = new AdcEvent( 1, pheader, pdataADC1, adc1size,
                                     serialNumberADC1 );
    if ( padc1->serial() == serialNumberADC1 ){
      padc1->ngf( pdataNGF1,ndataNGF1 );
      padc1->trg( pdataTRG1,ndataTRG1 );
      adc1List.push_back( padc1 );
      serialNumberADC1++;
    }
    else{
      delete  padc1;
      break;
    }
  }

#if DEBUG
  std::cout << "  ListSize :" 
            << " TDC0=" << tdc0List.size()
            << " TDC1=" << tdc1List.size()
            << " ADC0=" << adc0List.size()
            << " ADC1=" << adc1List.size()
            << std::endl;
#endif

  // SCALER bank
  DWORD*  pdataSCL;
  int  sclsize = bk_locate( pevent, "SCL0", &pdataSCL );
  if ( sclsize > 0 ){
    dumpScalerEvent( pheader,pevent );
  }

  //
  return CM_SUCCESS;
}

//
INT
event_builder_init()
{
  
  return CM_SUCCESS;
}

//
INT
event_builder_bor( INT run_number )
{
  // remove remained event list
  for( std::list<TdcEvent*>::iterator i = tdc0List.begin();
       i != tdc0List.end(); i++ ){
    delete *i;
  }
  tdc0List.clear();

  for( std::list<TdcEvent*>::iterator i = tdc1List.begin();
       i != tdc1List.end(); i++ ){
    delete *i;
  }
  tdc1List.clear();

  for( std::list<AdcEvent*>::iterator i = adc0List.begin();
       i != adc0List.end(); i++ ){
    delete *i;
  }
  adc0List.clear();

  for( std::list<AdcEvent*>::iterator i = adc1List.begin();
       i != adc1List.end(); i++ ){
    delete *i;
  }
  adc1List.clear();

  // Output file
  mfile.open( out_file, (out_gzip ? MIO_COMPRESS : 0) );
  mfile.bank( mbank );

  // message
  std::cout << "eventBuilder : Run " << run_number
            << std::endl;

  return CM_SUCCESS;
}

//
INT
event_builder_eor( INT run_number )
{
#if DEBUG
  std::cout << "event_builder_eor" << std::endl;
#endif

  // process remained events
  while( processEvent() )
    ;

  // remove remained event list
  for( std::list<TdcEvent*>::iterator i = tdc0List.begin();
       i != tdc0List.end(); i++ ){
    delete *i;
  }
  tdc0List.clear();

  for( std::list<TdcEvent*>::iterator i = tdc1List.begin();
       i != tdc1List.end(); i++ ){
    delete *i;
  }
  tdc1List.clear();

  for( std::list<AdcEvent*>::iterator i = adc0List.begin();
       i != adc0List.end(); i++ ){
    delete *i;
  }
  adc0List.clear();

  for( std::list<AdcEvent*>::iterator i = adc1List.begin();
       i != adc1List.end(); i++ ){
    delete *i;
  }
  adc1List.clear();
 
  // Close Output file
  mfile.close();

 //
  return CM_SUCCESS;
}

