// -*- C++ -*-
//
// FBmodule.cc
// Source file
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Apr 16 18:09:35 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include "fbcrate/FBcrate.h"


//
typedef struct{
  std::string   name;
  unsigned int  id;
  unsigned int  idmask;
} FB_MODULE;

typedef struct{
  std::string   name;
  std::string   commandname;
  std::string   csrdsr;
  unsigned int  value;
} FB_INIT;

static std::list<FB_MODULE>  s_module;
static std::list<FB_INIT>    s_init;


//
FBslot::FBslot( FBcrate* crate,
                int pa, 
                const std::string& name,
                const std::list<std::string>& init,
                const std::list<int>& channels )
  : m_crate( crate ),
    m_pa( pa ),
    m_name( name ),
    m_commandlist( init ),
    m_channellist( channels ),
    m_valid( false )
{
  //
}


//
FBslot::~FBslot()
{
  //
}


//
bool
FBslot::init()
{
  unsigned int  data,stat;
  bool  ret = false;
  m_valid = false;

  // check module
  std::list<FB_MODULE>::iterator m;
  for( m = s_module.begin(); m != s_module.end(); m++ ){
    if ( m->name == m_name ){
      m_crate->frc( m_pa, 0, &data );
      ret = true;
      //std::cout << std::hex << data << std::dec << std::endl;
      if ( (data & m->idmask) == m->id ){
        // send initialition command
        for( std::list<std::string>::const_iterator c = m_commandlist.begin();
             c != m_commandlist.end(); c++ ){
          m_valid = false;
          for( std::list<FB_INIT>::iterator i = s_init.begin();
               i != s_init.end(); i++ ){
            if ( i->name == m_name && i->commandname == *c ){
              m_valid = true;
              int  r = atoi( i->csrdsr.c_str() + 3 ); // CSR#/DSR#
              if ( i->csrdsr.c_str()[0] == 'C' ||
                   i->csrdsr.c_str()[0] == 'c'  ){ // CSR
                m_crate->fwc( m_pa, r, i->value );
                m_crate->frc( m_pa, r, &data );
                std::cout << "Slot" << std::setw(2) << m_pa
                          << " : CSR" << r
                          << " : stat=0x" 
                          << std::hex << std::setw(8) << std::setfill( '0' )
                          << data 
                          << std::dec << std::setw(0) << std::setfill( ' ' )
                          << std::endl;
              }
              else if ( i->csrdsr.c_str()[0] == 'D' ||
                        i->csrdsr.c_str()[0] == 'd'  ){ // DSR
                m_crate->fwd( m_pa, r, i->value );
                m_crate->frd( m_pa, r, &data );
                std::cout << "Slot" << std::setw(2) << m_pa
                          << " : DSR" << r
                          << " : stat=0x" 
                          << std::hex << std::setw(8) << std::setfill( '0' )
                          << data 
                          << std::dec << std::setw(0) << std::setfill( ' ' )
                          << std::endl;
              }
              break;
            }
          }
          if ( !m_valid ){
            ret = false;
            std::cout << "Slot" << std::setw(2) << m_pa
                      << " : Init command " << *c  << " failed.\n"
                      << std::endl;
          }
        }
      }
      else{
        
        ret = false;
        m_crate->reset();
        std::cout << "Slot" << std::setw(2) << m_pa
                  << " is not a valid module."
                  << std::endl;
      }
      break;
    }
  }
  if ( m == s_module.end() ){
    ret = false;
    std::cout << "No such module in datafile : " 
              << m_name
              << std::endl;
  }

  //
  return( ret );
}


//
FBcrate::FBcrate()
  : m_frc( NULL ),
    m_fwc( NULL ),
    m_frd( NULL ),
    m_fwd( NULL ),
    m_reset( NULL )
{
  //
}


//
FBcrate::~FBcrate()
{
  //
}


//
bool
FBcrate::init()
{
  if ( m_frc == NULL || m_frd == NULL ||
       m_fwc == NULL || m_fwd == NULL || 
       m_reset == NULL ){
    std::cout << "FBcrate::init : "
              << "frc,frd,fwc,fwd,reset are not correctly set." 
              << std::endl;
    return( false );
  }

  // start
  bool  ret = true;
  for( std::list<FBslot>::iterator s = m_slot.begin();
       s != m_slot.end(); s++ ){
    if ( !s->init() )
      ret = false;
  }
  
  return( ret );
}


//
int
FBcrate::readConf( const std::string& filename )
{
  std::ifstream  ifile( filename.c_str() );
  if ( ifile.good() ){
    while( !ifile.eof() ){
      char  buf[1024];
      ifile.getline( buf,1024 );
      char* next = buf;
      next = strtok( next," \n" );
      if ( next != NULL ){
        if ( next[0] == '#' ){
          continue; // comment words
        }
        else if ( strcmp( next,"module" ) == 0 ){
          //std::cerr << "M";
          FB_MODULE  m;
          if ( next = strtok( NULL," \n" ) )
            m.name = next;
          if ( next = strtok( NULL," \n" ) )
            m.id = strtoll( next,NULL,0 );
          if ( next = strtok( NULL," \n" ) )
            m.idmask = strtoll( next,NULL,0 );
          if ( next != NULL )
            s_module.push_back( m );
        }
        else if ( strcmp( next,"init" ) == 0 ){
          //std::cerr << "I";
          FB_INIT  m;
          if ( next = strtok( NULL," \n" ) )
            m.name = next;
          if ( next = strtok( NULL," \n" ) )
            m.commandname = next;
          if ( next = strtok( NULL," \n" ) )
            m.csrdsr = next;
          if ( next = strtok( NULL," \n" ) )
            m.value  = strtoll( next,NULL,0 );
          if ( next != NULL )
            s_init.push_back( m );
        }
        else if ( strcmp( next,"slot" ) == 0 ){
          //std::cerr << "S";
          int  slot = 0;
          std::string  name;
          std::list<std::string>  com;
          std::list<int>  channels;
          if ( next = strtok( NULL," \n" ) )
            slot = strtoll( next,NULL,0 );
          if ( next = strtok( NULL," \n" ) )
            name = next;
          if ( next = strtok( NULL," \n" ) ){
            char*  mm;
            while( mm = strchr( next,',' ) ){
              *mm = '\0';
              com.push_back( next );
              next = mm+1;
            }
            com.push_back( next );
          }
          while( next = strtok( NULL," ,\n" ) ){
            if ( strchr( next,'-' ) ){
              char*  mm = strchr( next,'-' );
              *mm = '\0';
              int  ni = strtoll( next,NULL,0 );
              int  ne = strtoll( mm+1,NULL,0 );
              for( int i=ni;i<=ne;i++ ){
                channels.push_back( i );
              }
            }
            else{
              channels.push_back( strtoll( next,NULL,0 ) );
            }
          }
          if ( !com.empty() )
            m_slot.push_back( FBslot( this,slot,name,com,channels ) );
        }
      }
    }
    std::cerr << std::endl;
  }
  return( 0 );
}


void
FBcrate::dumpConf()
{
  // modules
  for( std::list<FB_MODULE>::iterator i = s_module.begin();
       i != s_module.end(); i++ ){
    std::cout << "module"
              << std::setw(16) << i->name 
              << "  0x" 
              << std::hex << std::setw(8) << std::setfill( '0' )
              << i->id
              << "  0x" 
              << i->idmask
              << std::dec << std::setw(0) << std::setfill( ' ' )
              << std::endl;
  }
  // init
  for( std::list<FB_INIT>::iterator i = s_init.begin();
       i != s_init.end(); i++ ){
    std::cout << "init  "
              << std::setw(16) << i->name 
              << std::setw(16) << i->commandname
              << std::setw(8)  << i->csrdsr 
              << "  0x" 
              << std::hex << std::setw(8) << std::setfill( '0' )
              << i->value
              << std::dec << std::setw(0) << std::setfill( ' ' )
              << std::endl;
  }
  // slot
  for( std::list<FBslot>::iterator i = m_slot.begin();
       i != m_slot.end(); i++ ){
    std::cout << "slot  "
              << std::setw(2)  << i->pa()
              << std::setw(16) << i->name() << "  "
              << std::dec << std::setw(0) << std::setfill( ' ' )
              << "  init=";
    for( std::list<std::string>::const_iterator j = i->commandList().begin();
         j != i->commandList().end(); j++ ){
      std::cout << *j << ",";
    }
    std::cout << "  read=";
    for( std::list<int>::const_iterator j = i->channelList().begin();
         j != i->channelList().end(); j++ ){
      std::cout << *j << ",";
    }
    std::cout << std::endl;
  }  
}


