// -*- C++ -*-
//
// k2917_camac.cc
// utility
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul  4 16:42:13 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <list>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "midas.h"
#include "mcstd.h"

#include "k2917/K2917.h"


//
static int    debugLevel = 0;

// START flag
static bool  Started = false;

// K2917
static const unsigned int  K2917_BASEADDR = 0xff00;

static K2917  kcam( K2917_BASEADDR );

// CAMAC modules
typedef struct{
  int             slot;
  std::list<int>  channels;
} CAMAC_MODULE;
std::list<CAMAC_MODULE>  camacmodule;


////////////////
static int
readCamacConf( const std::string& filename )
{
  std::ifstream  ifile( filename.c_str() );
  if ( ifile.good() ){
    while( !ifile.eof() ){
      char  buf[1024];
      ifile.getline( buf,1024 );
      char*  next = buf;
      next = strtok( next," \n" );
      if ( next != NULL ){
        if ( next[0] == '#' ){
          continue; // comment words
        }
        else if ( strcmp( next,"slot" ) == 0 ){
          CAMAC_MODULE  cm;
          if ( next = strtok( NULL," \n" ) )
            cm.slot = strtoll( next,NULL,0 );
          while( next = strtok( NULL," ,\n" ) ){
            if ( strchr( next,'-' ) ){
              char*  mm = strchr( next,'-' );
              *mm = '\0';
              int  ni = strtoll( next,NULL,0 );
              int  ne = strtoll( mm+1,NULL,0 );
              for( int i=ni;i<=ne;i++ ){
                cm.channels.push_back( i );
              }
            }
            else{
              cm.channels.push_back( strtoll( next,NULL,0 ) );
            }
          }
          camacmodule.push_back( cm );
        }
      }
    }
    // dump
    std::cout << "CAMAC modules" << std::endl;
    for( std::list<CAMAC_MODULE>::iterator i=camacmodule.begin();
         i!=camacmodule.end();i++ ){
      std::cout << "slot " << std::setw(2) << i->slot
                << " : ";
      for( std::list<int>::iterator a=i->channels.begin();
           a!=i->channels.end();a++ ){
        std::cout << std::setw(2) << *a << " ";
      }
      std::cout << std::endl;
    }
  }
  return( camacmodule.size() );
}


////////////////
void
initK2917Camac()
{
  if ( debugLevel )
    std::cerr << "initK2917Camac" << std::endl;

  readCamacConf( "CAMACcrate.conf" );

}


////////////////
void
termK2917Camac()
{
  if ( debugLevel )
    std::cerr << "termNgf" << std::endl;
  
}


////////////////
void
beginRunK2917Camac()
{
  if ( debugLevel )
    std::cerr << "beginRunK2917Camac" << std::endl;
  
  // Wait until all frontends are ready.
  kcam.camac_init();
  Started = false;
}


////////////////
void
endRunK2917Camac()
{
  if ( debugLevel )
    std::cerr << "endRunK2917Camac" << std::endl;
  
  // stop
  Started = false;
}


////////////////
void
pauseRunK2917Camac()
{
  if ( debugLevel )
    std::cerr << "pauseRunK2917Camac" << std::endl;
 
  // stop
  Started = false;
}


////////////////
void
resumeRunK2917Camac()
{
  if ( debugLevel )
    std::cerr << "resumeRunK2917Camac" << std::endl;
 
  // Wait until all frontends are ready.
  Started = false;
}


////////////////
void
fillBankK2917Camac( char*& pevent )
{
  if ( debugLevel )
    std::cerr << "fillBankK2917Camac" << std::endl;
  
  // Only in the start sequence.
  if ( !Started ){
    Started = true;
    //std::cerr << "K2917-CAMAC started" << std::endl;
    return;
  }

  // create SCALER bank
  DWORD *pdata;
  bk_create(pevent, "SCL0", TID_DWORD, &pdata);

  unsigned int  data = 0;
  for( std::list<CAMAC_MODULE>::iterator m=camacmodule.begin();
       m!=camacmodule.end(); m++ ){
    for( std::list<int>::iterator a=m->channels.begin();
         a!=m->channels.end(); a++ ){
      kcam.camac( NAF(m->slot,*a,0),&data );
      *pdata++ = data;
    }
  }
  
  // close SCALER bank
  bk_close(pevent, pdata);
  
}
