// -*- C++ -*-
//
// ngf_1885n.cc
// utility
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Mon Apr  8 21:14:27 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "midas.h"
#include "mcstd.h"

#include "ngf/Ngf.h"
#include "fbcrate/FBcrate.h"
#include "vmeioreg/VmeIoreg.h"
#include "ngf_1885n.h"

//
static int    debugLevel = 0;

// START flag
static bool  Started = false;

// POLLING
bool    PollTrigger = false;

// For 1885N : High-range or Auto range
#define ADCRANGE_AUTO     0x0
#define ADCRANGE_LOW      0x1
#define ADCRANGE_HIGH     0x2
#define ADCRANGE_SWITCH   0x4
#define ADCRANGE_REVERSE  0x8
#define LRS1885N_AUTORANGE  0x00001000
#define LRS1885N_HIGHRANGE  0x10002000
#define LRS1885N_LOWRANGE   0x30000000
static int  AdcRange        = ADCRANGE_AUTO;
static int  RangePeriod1    = 0; // #spills. For MIX mode
static int  RangePeriod2    = 0; // #spills. For MIX mode

static int  CurRange  = 0;
static int  CurPeriod = 0;
static bool ChangeAdcRangeFlag = false;

// NGF
static const unsigned int  NGF_BASEADDR = 0x80e00000;
Ngf  ngf( NGF_BASEADDR );

// FASTBUS crate and modules
FBcrate  fbcrate;

// Data buffer
typedef struct{
  int            event_number;
  unsigned int   trigger;
  unsigned int   time;
  unsigned int   status[8];
  int            data_size;
  unsigned int*  buffer;
} DATABUF;

static int  Nevent  = 0;
static int  Nstatus = 0;
static int  MaxEvent = 1000; // dummy
static bool Overflow = false;
static int  DataSize = 0;
static DATABUF*  databuf;

// VME-IOREG
static const unsigned int  VIOREG_BASEADDR  = 0x8000;
VmeIoreg  vioreg( VIOREG_BASEADDR );


////////////////
// Interrupt Handler
bool  Interrupted = false;
extern"C"
void
sighdl( int arg )
{
  Interrupted = true;
  
  if ( debugLevel ){
    std::cerr << "Interrupt received. arg=" << arg << std::endl;
  }
}


// Required functions for FBcrate class
void
frc( unsigned int pa,unsigned int sa,unsigned int* data )
{
  ngf.frc( pa,sa,data );
}

void
fwc( unsigned int pa,unsigned int sa,unsigned int data )
{
  ngf.fwc( pa,sa,data );
}

void
frd( unsigned int pa,unsigned int sa,unsigned int* data )
{
  ngf.frd( pa,sa,data );
}

void
fwd( unsigned int pa,unsigned int sa,unsigned int data )
{
  ngf.fwd( pa,sa,data );
}

void
reset()
{
  ngf.SeqResetReg( 0 );
  ngf.SeqEnableReg( 0 );
}


////////////////
void
initSettings()
{
  // set special keys
  int  size = sizeof(INT);
  HNDLE  hDB;
  cm_get_experiment_database(&hDB, NULL);
  db_create_record( hDB, 0, "/Equipment/NGF2-ADC/Settings",
                    "AdcRange = INT : 0\n"
                    "RangePeriod1 = INT : 3\n"
                    "RangePeriod2 = INT : 897"
                    );
}


////////////////
static void
getAdcRange()
{
  int  size = sizeof(INT);
  HNDLE  hDB;
  cm_get_experiment_database(&hDB, NULL);
  //
  if ( db_get_value( hDB,0, "/Equipment/NGF2-ADC/Settings/AdcRange", 
                     &AdcRange, &size, TID_INT, TRUE) == DB_SUCCESS  ){
    int  sw  = AdcRange & ADCRANGE_SWITCH;
    int  rev = AdcRange & ADCRANGE_REVERSE;
    int  rg  = AdcRange & 0x03;
    if ( !sw ){
      if ( rg == ADCRANGE_AUTO )
        std::cerr << std::endl << "ADC range is Auto          " << std::endl;
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is High          " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Low           " << std::endl;
      else
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
    }
    else if ( !rev ){
      if ( rg == ADCRANGE_AUTO ){
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
        sw = 0;
      }
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is High+Auto     " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Low+Auto      " << std::endl;
      else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) )
        std::cerr << std::endl << "ADC range is High+Low      " << std::endl;
    }
    else { // sw + rev
      if ( rg == ADCRANGE_AUTO ){
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
        sw = 0;
      }
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is Auto+High     " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Auto+Low      " << std::endl;
      else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) )
        std::cerr << std::endl << "ADC range is Low+High      " << std::endl;
    }
  }
  else{
    std::cerr << "Cannot find AdcRange key." << std::endl;
  }
  
  //
  if ( AdcRange & ADCRANGE_SWITCH ){
    if ( db_get_value( hDB,0, "/Equipment/NGF2-ADC/Settings/RangePeriod1", 
                       &RangePeriod1, &size, TID_INT, TRUE) == DB_SUCCESS ){
      std::cerr << "RangePeriod1 is " << RangePeriod1 << " spill(s)   " 
                << std::endl;
    }
    else{
      std::cerr << "Cannot find RangePeriod1 key." << std::endl;
    }
    
    if ( db_get_value( hDB,0, "/Equipment/NGF2-ADC/Settings/RangePeriod2", 
                       &RangePeriod2, &size, TID_INT, TRUE) == DB_SUCCESS ){
      std::cerr << "RangePeriod2 is " << RangePeriod2 << " spill(s)   " 
                << std::endl;
    }
    else{
      std::cerr << "Cannot find RangePeriod2 key." << std::endl;
    }
    
  }
}
  

////////////////
static void
changeAdcRange()
{
  int  sw  = AdcRange & ADCRANGE_SWITCH;
  int  rev = AdcRange & ADCRANGE_REVERSE;
  int  rg  = AdcRange & 0x03;

  unsigned int  rangebit;
    
  if ( rg == ADCRANGE_HIGH ){ // high-auto or auto-high
    if ( CurRange == ADCRANGE_AUTO || CurRange == -1 ){
      rangebit  = LRS1885N_HIGHRANGE;
      CurRange  = ADCRANGE_HIGH;
      CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
    }
    else{
      rangebit  = LRS1885N_AUTORANGE;
      CurRange  = ADCRANGE_AUTO;
      CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
    }
  }
  else if ( rg == ADCRANGE_LOW ){ // low-auto or auto-low
    if ( CurRange == ADCRANGE_AUTO || CurRange == -1 ){
      rangebit  = LRS1885N_LOWRANGE;
      CurRange  = ADCRANGE_LOW;
      CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
    }
    else{
      rangebit  = LRS1885N_AUTORANGE;
      CurRange  = ADCRANGE_AUTO;
      CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
    }
  }
  else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) ){ // high-low or low-high
    if ( sw ){
      if ( CurRange == ADCRANGE_LOW || CurRange == -1 ){
        rangebit  = LRS1885N_HIGHRANGE;
        CurRange  = ADCRANGE_HIGH;
        CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
      }
      else{
        rangebit  = LRS1885N_LOWRANGE;
        CurRange  = ADCRANGE_LOW;
        CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
      }
    }
    else{
      if ( CurRange == -1 ){
        rangebit  = LRS1885N_AUTORANGE;
        CurRange  = ADCRANGE_AUTO;
        CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
      }
    }
  }
  
  //
  ngf.disableList();
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->name() == "LRS1885N" ){
      ngf.fwc( s->pa(),0,rangebit );
#if 0
      unsigned int  data;
      ngf.frc( s->pa(),0,&data );
      std::cout << std::hex << data << std::dec << std::endl;
      std::cout << std::hex << ngf.SeqStatusReg() << std::dec << std::endl;
#endif        
    }
  }
  ngf.enableList();

#if 0
  std::cout << "ADCrange=" << std::hex << CurRange << std::dec << std::endl;
#endif

  ChangeAdcRangeFlag = false;
}


////////////////
void
initNgf()
{
  if ( debugLevel )
    std::cerr << "initNgf" << std::endl;

  // Use Interrupt for trigger event?
  if ( getenv( "POLL_TRIGGER" ) ){
    PollTrigger = true;
    std::cout << std::endl
              << "POLL_TRIGGER environment variable is set."
              << std::endl;
  }
  else{
    PollTrigger = false;
    std::cout << std::endl
              << "POLL_TRIGGER environment variable is NOT set."
              << std::endl;
  }
  
  // Disable interrupt
  vioreg.disableInt();
  vioreg.clear();
  vioreg.mask();
  vioreg.level_out( 0x00 );
  
  // VME interrupt setup
  if ( !VmeIoreg::initVuiInterrupt( 0,5,0x50 ) ){
    abort();
  }
  sigset( SIGINT,sighdl );

  // NGF
  ngf.SeqResetReg( 0 );
  ngf.FbTimeoutReg( 0x73 );
  ngf.SeqEnableReg( 0 );
  ngf.maskInterrupt( NGF_CLEAR_INT_ALL ); // disable all

  // FBmodule
  fbcrate.readConf( "FBcrate.conf" );
  fbcrate.setFunc( frc,fwc,frd,fwd,reset );
  fbcrate.init();

}

  
////////////////
void
initNgfCommandList()
{
  if ( debugLevel )
    std::cerr << "initNgfCommandList" << std::endl;

  std::cout << "Loading command list : ";
  std::list<NgfList>  ls;
  ls.push_back( NgfList( NGF_WAIT_SEQ_GO_FLAG,0 ) );
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,NGF_OUT_NIM1 ) );
  
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() ){
      ls.push_back( NgfList( FB_PRIM_DSR,s->pa() ) );
      for( std::list<int>::const_iterator sa = s->channelList().begin();
           sa != s->channelList().end(); sa++ ){
        if ( s->name() != "LRS1885N" )
          ls.push_back( NgfList( FB_SECAD_W,*sa ) );
        ls.push_back( NgfList( FB_DATA_R,0 ) );
      }
      ls.push_back( NgfList( FB_DISCON,0 ) );
    }
  }
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,
                         NGF_CLEAR_NIM1 | NGF_OUT_NIM2 | NGF_OUT_NIM3 ) );
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,
                         NGF_CLEAR_NIM2 | NGF_CLEAR_NIM3 ) );
  ls.push_back( NgfList( NGF_CLEAR_SEQ_GO_FLAG,0 ) );
  ls.push_back( NgfList( NGF_ENABLE_RAM_SEQUENCER,0 ) );
  //
  ngf.setList( 0x0000, ls );
  //
  std::cout << ls.size() << " sequence." << std::endl;

  // Data Size
  DataSize = 0;
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() )
      DataSize += s->channelList().size();
  }
  std::cout << "Data size = " << DataSize << " channels/Event." << std::endl;

  // Data Buffer
  extern INT  max_event_size; // in frontend.cc, fe2.cc
  int  evsize = 1 + DataSize + 1 + 1; //header,terminator
  MaxEvent = max_event_size / 4 / evsize;
  databuf = new DATABUF[MaxEvent];
  if ( databuf == NULL ){
    perror( "Cannot allocate databuf" );
    abort();
  }
  for( int i=0;i<MaxEvent;i++ ){
    databuf[i].buffer = new unsigned int[DataSize];
    if ( databuf[i].buffer == NULL ){
      perror( "Cannot allocate databuf" );
      abort();
    }
  }
  std::cout << "MaxEvent = " << MaxEvent << "/spill." << std::endl;
}


////////////////
void
termNgf()
{
  if ( debugLevel )
    std::cerr << "termNgf" << std::endl;

  ngf.disableList();
  
  vioreg.disableInt();
  
  // close
  VmeIoreg::termVuiInterrupt();
  sigset( SIGINT,SIG_DFL );

  //
  for( int i=0;i<MaxEvent;i++ )
    delete databuf[i].buffer;
  delete [] databuf;
}


////////////////
void
beginRunNgf()
{
  if ( debugLevel )
    std::cerr << "beginRunNgf" << std::endl;
  
  // For 1885N : High-range or Auto range
  getAdcRange();
  CurRange  = -1;
  changeAdcRange();
  if ( AdcRange & ADCRANGE_REVERSE )
    changeAdcRange(); // again
  CurPeriod++; // for dummy spill
#if 0
  std::cout << "ADCrange=" << std::hex << CurRange << std::dec << std::endl;
#endif
  
  // Enable NGF list sequencer
  ngf.enableList();
  
  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );
  //std::cerr << "Waiting other frontends ready." << std::endl;

  // Enable interrupt
  Nevent = 0;
  Nstatus = 0;
  Overflow = false;
  Interrupted = false;
  if ( PollTrigger ){
    vioreg.enableInt(1);
  }
  else{
    vioreg.enableInt(1);
    vioreg.enableInt(2);
  }

}


////////////////
void
endRunNgf()
{
  if ( debugLevel )
    std::cerr << "endRunNgf" << std::endl;

  // Disable NGF list sequencer
  ngf.disableList();

  // Disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // stop
  Started = false;
}


////////////////
void
pauseRunNgf()
{
  if ( debugLevel )
    std::cerr << "pauseRunNgf" << std::endl;
 
  // Disable NGF list sequencer
  ngf.disableList();
  
  // Disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // stop
  Started = false;
}


////////////////
void
resumeRunNgf()
{
  if ( debugLevel )
    std::cerr << "resumeRunNgf" << std::endl;
 
  // Enable NGF list sequencer
  ngf.enableList();
  
  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );

  // Enable interrupt
  Nevent = 0;
  Nstatus = 0;
  Overflow = false;
  Interrupted = false;
  if ( PollTrigger ){
    vioreg.enableInt(1);
  }
  else{
    vioreg.enableInt(1);
    vioreg.enableInt(2);
  }

}


//#define CLEARFIRST

////////////////
void
bufferEventNgf()
{
  if ( debugLevel )
    std::cerr << "bufferEventNgf" << std::endl;
  
  // Start reading from FIFO.
  // Event time
  unsigned int  eventtime = time(NULL);
  // read trigger info
  unsigned int  trig = vioreg.input_flipflop() & 0xff;
  if ( CurRange == ADCRANGE_HIGH )
    trig |= 0x8000;
  else if ( CurRange == ADCRANGE_LOW )
    trig |= 0x4000;
#ifdef CLEARFIRST
  // AdcRange
  if ( ChangeAdcRangeFlag )
    changeAdcRange();
  // accept next event
  Interrupted = false;
  vioreg.clear(2);// Trigger signal
  vioreg.clear(3);// Trigger info
  vioreg.out( 0x0f );
#endif
  ngf.VmeOutReg( NGF_OUT_NIM3 );
  // NGF status
  Nstatus = 0;
  
  // read each trigger event
  if ( Nevent < MaxEvent ){
    databuf[Nevent].event_number = Nevent;
    databuf[Nevent].trigger = trig;
    databuf[Nevent].time = eventtime;
    databuf[Nevent].data_size = DataSize;
    //
    databuf[Nevent].status[Nstatus++] = ngf.SeqFifoAndEclNimInputReg();
    databuf[Nevent].status[Nstatus++] = ngf.FbStatusReg1();
    databuf[Nevent].status[Nstatus++] = ngf.FbStatusReg2();
    //
    while( 1 ){
      //std::cerr << "TRG=" << std::hex << databuf[Nevent].trigger 
      //          << std::dec << "\n";

      //int  stat = ngf.readFifo( DataSize, databuf[Nevent].buffer );
      int  stat = ngf.readFifoNoWait(DataSize,databuf[Nevent].buffer );
      if ( debugLevel )
        std::cout << "bufferEventNgf : stat=0x" 
                  << std::setw(8) << std::setfill('0') << std::hex
                  << stat
                  << std::dec << std::setw(0) << std::endl;
      if ( stat & 0xf0 ){ // sequence failed due to module busy
        std::cerr << "\rbufferEventNgf : module busy : SeqStat=0x" 
                  << std::setw(8) << std::setfill('0') << std::hex
                  << stat
                  << " FBStat=0x" 
                  << ngf.FbStatusReg1()
                  << ",0x" 
                  << ngf.FbStatusReg2()
                  << std::dec << std::setw(0) << std::endl;
        ngf.enableList();
        std::cerr << "bufferEventNgf : Ngf::enableList called" << std::endl;
      }
      else{
        break;
      }
    }
    //
    databuf[Nevent].status[Nstatus++] = ngf.SeqStatusReg();
    //
    Nevent++;
  }
  else{
    if ( !Overflow ){
      std::cerr << "\rbufferEventNgf : Nevent exceeds the max size "
                << MaxEvent << std::endl;
      Overflow = true;
    }
  }

  // End of reading from FIFO.
  ngf.VmeOutReg( NGF_CLEAR_NIM3 );

#ifndef CLEARFIRST
  // AdcRange
  if ( ChangeAdcRangeFlag )
    changeAdcRange();
  // accept next event
  Interrupted = false;
  vioreg.clear(2);// Trigger signal
  vioreg.clear(3);// Trigger info
  vioreg.out( 0x0f );
#endif
}


////////////////
void
fillBankNgf( char* pevent )
{
  if ( debugLevel )
    std::cerr << "fillBankNgf" << std::endl;
  
  // Only in the start sequence.
  if ( !Started ){
    vioreg.level_out( 0x00 );
    vioreg.out( 0x01 );
    Started = true;
    //
    return;
  }
  
  // create NFG0 bank
  DWORD  *pdata;
  bk_create(pevent, "NGF1", TID_DWORD, &pdata);
  *pdata++ = Nevent | (Overflow ? (0x01 << 31) : 0 );
  *pdata++ = Nstatus;
  for( int i=0; i<Nevent; i++ ){
    for( int j=0; j<Nstatus; j++ ){
      *pdata++ = databuf[i].status[j];
    }
  }
  bk_close(pevent, pdata);

  // create TRG1 bank
  bk_create(pevent, "TRG1", TID_DWORD, &pdata);
  for( int i=0; i<Nevent; i++ ){
    *pdata++ = (((databuf[i].event_number+1)<<16) | databuf[i].trigger);
  }
  bk_close(pevent, pdata);
  
  // create ADC bank
  bk_create(pevent, "ADC1", TID_DWORD, &pdata);

  for( int i=0; i<Nevent; i++ ){
    *pdata++ = (((databuf[i].event_number+1)<<16) | (databuf[i].data_size+1));
    for( int j=0; j<DataSize; j++ ){
    //for( int j=0; j<6; j++ ){
      *pdata++ = databuf[i].buffer[j];
    }
    *pdata++ = 0xff000000;
  }
  if ( Nevent > 0 )
    *pdata++ = 0xffffffff;

  // ADC Range
  if ( AdcRange & ADCRANGE_SWITCH && --CurPeriod <= 0 )
    ChangeAdcRangeFlag = true;

  // close ADC bank
  bk_close(pevent, pdata);

  // clear buffer for next event
  Nevent = 0;
  Overflow = false;
  Interrupted = false;
  vioreg.clear(1);// Spill signal

  ngf.VmeOutReg( NGF_OUT_NIM4 );
  ngf.VmeOutReg( NGF_CLEAR_NIM4 );
}

