// -*- C++ -*-
//
// NGFlib.h
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 25 22:32:13 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#ifndef NGFLIB_H_INCLUDED
#define NGFLIB_H_INCLUDED

// includes
#include <list>
#include <vector>
#include <sys/vme.h>

//////
class NgfList{
public:
  NgfList():func(0),data(0){};
  NgfList( unsigned int f, unsigned int d ):func(f),data(d){}
  ~NgfList(){}
  unsigned int  func;
  unsigned int  data;
};


////// class 
class Ngf {
public:
  // constructor/destructor
  Ngf();
  Ngf( unsigned int baseaddr, int debugLevel = 0 );
  Ngf( const Ngf& );
  ~Ngf();
  // operator
  Ngf&    operator=( const Ngf& );
  // utilities
  bool          init( unsigned int baseaddr, int debugLevel = 0 );
  void          command( unsigned int reg, unsigned int cmd );
  unsigned int  status( unsigned int reg );
  const unsigned int*  map() { return m_map; }
  const unsigned int   slaveAddr() { return m_slaveaddr; }
  const unsigned int*  slavePtr()  { return (unsigned int*)m_slaveptr; }

  // NGF registers
  // read
  inline unsigned int  InternalFbIoReg();
  inline unsigned int  FbLastPrimaryAddressReg();
  inline unsigned int  FbTimeoutReg();
  inline unsigned int  FbArbitrationLevelReg();
  inline unsigned int  FbProtocolReg();
  inline unsigned int  SeqFifoAndEclNimInputReg();
  inline unsigned int  VmeIrqReg();
  inline unsigned int  VmeIrqMaskReg();
  inline unsigned int  NextSeqRamAddressReg();
  inline unsigned int  LastSeqProtocolReg();
  inline unsigned int  SeqStatusReg();
  inline unsigned int  FbStatusReg1();
  inline unsigned int  FbStatusReg2();
  inline unsigned int  PedestalMemoryPointerReg();
  inline unsigned int  PedestalRemapReg();
  // write
  inline void          VmeOutReg( unsigned int );
  inline void          ClearVmeOutReg( unsigned int );
  inline void          InternalAuxPortReg( unsigned int );
  inline void          GenerateAuxB40PulseReg( unsigned int );
  inline void          FbTimeoutReg( unsigned int );
  inline void          FbArbitrationLevelReg( unsigned int );
  inline void          VmeIrqReg( unsigned int );
  inline void          VmeIrqMaskReg( unsigned int );
  inline void          NextSeqRamAddressReg( unsigned int );
  inline void          ResetReg( unsigned int );
  inline void          SeqEnableReg( unsigned int );
  inline void          SeqDisableReg( unsigned int );
  inline void          SeqRamLoadEnableReg( unsigned int );
  inline void          SeqRamLoadDisableReg( unsigned int );
  inline void          SeqResetReg( unsigned int );
  inline void          ClearSeqCmdFlagReg( unsigned int );
  inline void          EnablePedestalSubtractionReg( unsigned int );
  inline void          DisablePedestalSubtractionReg( unsigned int );
  inline void          PedestalMemoryPointerReg( unsigned int );
  inline void          PedestalRemapReg( unsigned int );
  // Interrupt
  inline void          initInterrupt( int level, int vector );
  inline void          maskInterrupt( int src );
  inline void          enableInterrupt();
  inline void          disableInterrupt();
  // Sequencer
  inline unsigned int  Seq2VmeFifo( unsigned int offset );
  inline void          Vme2SeqFifo( unsigned int offset, unsigned int data );
  unsigned int         readFifo( int size, unsigned int* data );
  unsigned int         readFifoNoWait( int size, unsigned int* data );
  // List
  unsigned int  setList( unsigned int addr, const std::list<NgfList>& ls );
  unsigned int  setList( unsigned int addr, const std::vector<NgfList>& ls );
  unsigned int  setList( unsigned int addr, NgfList ls[], int size );
  unsigned int  enableList();
  unsigned int  disableList();
  // FASTBUS single action
  unsigned int  frc( int pa,int sa, unsigned int* data );
  unsigned int  fwc( int pa,int sa, unsigned int data );
  unsigned int  frd( int pa,int sa, unsigned int* data  );
  unsigned int  fwd( int pa,int sa, unsigned int data );
  unsigned int  frdb( int pa, int sa, unsigned int mode,
                      unsigned int limitcounter, unsigned int buf,
                      unsigned int* recword, unsigned int* nextbuf );
  unsigned int  frdbm( int pa, int sa, unsigned int mode,
                       unsigned int limitcounter, unsigned int buf,
                       unsigned int* recword, unsigned int* nextbuf );

  unsigned int  frdb_single( int pa,int n_sa,int sa[],unsigned int data[] );
  unsigned int  frdb_single( int pa,int n_sa,unsigned int data[] );

  unsigned int  waitSeqDoneR( unsigned int* data );
  unsigned int  waitSeqDoneW();
private:
  // data
  int            m_debugLevel;
  unsigned int   m_baseaddr;
  int            m_fd32;
  unsigned int*  m_map;

  ioctl_map_t    m_dvma;
  int            m_fdslave;
  caddr_t        m_slaveptr;
  unsigned int   m_slaveaddr;

  // utilities
  bool          term();
};


// inline function
#include "Ngf.icc"


//////
#endif // NGFLIB_H_INCLUDED

