// -*- C++ -*-
//
// VmeVisualScaler.cc
// example code
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu May 23 22:38:26 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/vme.h>
#include "vmevscaler/VmeVisualScaler.h"


//////
#define DEBUG 0


//////
// static members


//////
// constants
static const unsigned int  VMEVSCALER_MAPSIZE = 0x100;


//////
inline int
addrindex( int addr )
{
  return( addr >> 1 );
}


//////
VmeVisualScaler::VmeVisualScaler()
  : m_debugLevel( 0 ),
    m_fd24( -1 ),
    m_baseaddr( 0 ),
    m_mapsize( 0 ),
    m_mapbase( (unsigned short*)-1 ),
    m_map(     (unsigned short*)-1 )
{
#if DEBUG
  std::cout << "VmeVisualScaler::VmeVisualScaler()" << std::endl;
#endif
}


//////
VmeVisualScaler::VmeVisualScaler( unsigned int baseaddr, int debugLevel )
  : m_debugLevel( debugLevel ),
    m_fd24( -1 ),
    m_baseaddr( baseaddr ),
    m_mapsize( 0 ),
    m_mapbase( (unsigned short*)-1 ),
    m_map(     (unsigned short*)-1 )
{
#if DEBUG
  std::cout << "VmeVisualScaler::VmeVisualScaler(addr)" << std::endl;
#endif

  // initialize
  if ( !init( m_baseaddr ) ){
    std::cout << "VmeVisualScaler::VmeVisualScaler(addr) : init failed" 
              << std::endl;
    exit( 1 );
  }

};


//////
VmeVisualScaler::~VmeVisualScaler()
{
#if DEBUG
  std::cout << "VmeVisualScaler::~VmeVisualScaler()" << std::endl;
#endif
  
  if ( (caddr_t)m_mapbase != (caddr_t)-1 ){
    munmap( (char*)m_mapbase,m_mapsize );
  }
  
  if ( m_fd24 >= 0 ){
    close( m_fd24 );
  }
  
}


//////
bool
VmeVisualScaler::init( unsigned int baseaddr )
{
#if DEBUG
  std::cerr << "VmeVisualScaler::init()" << std::endl;
#endif

  // open VME device file 
  m_fd24 = open( "/dev/vme24d32",O_RDWR );
  if ( m_fd24 == -1 ){
    perror( "VmeVisualScaler::init : open /dev/vme24d32" );
    return( false );
  }

  // base addr
  m_baseaddr = baseaddr;
  unsigned int  mbase = m_baseaddr & 0xffffe000; // pagesize = 8192 = 0x2000
  
  // mmap
  m_mapsize = VMEVSCALER_MAPSIZE + m_baseaddr - mbase;
  m_mapbase = (unsigned short*)mmap( 0, m_mapsize,
                                     PROT_WRITE|PROT_READ,MAP_SHARED,
                                     m_fd24, mbase );
  m_map = (unsigned short*)((unsigned int)m_mapbase + m_baseaddr - mbase);
  if ( m_debugLevel )
    std::cout << "VmeVisualScaler::init : mapbase=" << m_mapbase
              << " : map=" << m_map 
              << std::hex
              << " : baseaddr=" << m_baseaddr
              << " : mapsize=" << m_mapsize
              << " : diff=" << (int)m_map - (int)m_mapbase
              << std::dec
              << std::endl;
  if ( (caddr_t)m_mapbase  == (caddr_t)-1 ){
    perror( "VmeVisualScaler::init : mmap" );
    return( false );
  }

  return( true );
}


//////
void
VmeVisualScaler::reset()
{
  m_map[addrindex(VMEVSCALER_RESET)];
}


//////
int
VmeVisualScaler::read( int ch )
{
  return( m_map[addrindex(VMEVSCALER_CH0 + ch)] );
}


