// -*- C++ -*-
//
// GsimHits.cc
// Detector hit recording function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Jul 11 15:29:57 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <math.h>
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/Ntuple.h"
#include "ntuple.h"


//////
#define  DEBUG  0


//////
// FFkey
float    DetEnergyThreshold = 100; // GeV

// GsimUserGeom.cc
extern int    DetUseCBAR[];
extern int    DetUseCS00[];
extern float  DetCS00[];
extern float  DetCSCI[];
extern float  DetCPB1[];
extern float  DetCPB2[];


//////
int
GsimHits()
{
#if DEBUG
  std::cerr << "GsimHits" << std::endl;
#endif

  // Return code. 0:Do not stop, >0:Stop swimming, <0:Stop Event_process
  int  stop = 0;

  // Check Volume
  std::string  dname( (char*)&gcvolu_.NAMES[gcvolu_.NLEVEL-1],4 );
#if DEBUG
  std::cerr << dname << " : edep=" << gctrak_.DESTEP << std::endl;
#endif

  int     pid  = gckine_.IPART;
  int     lv   = gcvolu_.NLEVEL - 1;
  int     id   = gcvolu_.NUMBER[lv]-1;
  int     moid = gcvolu_.NUMBER[lv-1]-1; // ID of mother volume
  double  ekin = gctrak_.GEKIN;
  double  etot = gctrak_.GETOT;
  double  edep = gctrak_.DESTEP;
  Hep3Vector  v( gctrak_.VECT[0],gctrak_.VECT[1],gctrak_.VECT[2] );
  Hep3Vector  p( gctrak_.VECT[3]*gctrak_.VECT[6],
                 gctrak_.VECT[4]*gctrak_.VECT[6],
                 gctrak_.VECT[5]*gctrak_.VECT[6] );

  // Central Barrel
  if ( dname == "CBAR" && DetUseCBAR[1]==0 ){
    nresult.ESCI[id] += ekin;
    stop = 2;
  }
  else if ( gctrak_.INWVOL == 3 ){
    //std::cerr << dname << " [" << id 
    //          << "] : escape : edep=" << ekin << std::endl;
    nresult.EESCAPE += ekin;
    nresult.ETOT    += ekin;
  }
  else{
    if ( dname == "CS00" ){
      nresult.ESCI00[moid] += edep;
      nresult.ES00SUM      += edep;
      nresult.ETOT         += edep;
    }
    else if ( dname == "CSUS" ){
      nresult.ESUS[moid] += edep;
      nresult.ESUSSUM    += edep;
      nresult.ETOT       += edep;
    }
    else if ( dname == "CSCI" ){
      nresult.ESCI[moid] += edep;
      nresult.ESCISUM    += edep;
      nresult.ETOT       += edep;
      // Energy threshold to stop event_process for speed_up.
      if ( nresult.ESCI[moid] > DetEnergyThreshold ){
        //std::cerr << dname << " [" << id 
        //          << "] : E=" << nresult.ESCI[moid]
        //          << " > Ethresh(" << DetEnergyThreshold << ")" 
        //          << std::endl;
        stop = -2; // Stop Event, see Gustep.cc
      }
    }
    else if ( dname == "CPBL" ){
      nresult.ELEAD[moid] += edep;
      nresult.ELEADSUM    += edep;
      nresult.ETOT        += edep;
    }
    else{
      //if ( edep > 0 ){
      //  std::cerr << dname << " [" << id 
      //            << "] : edep=" << gctrak_.DESTEP << std::endl;
      //}
      nresult.EOTHER += edep;
      nresult.ETOT   += edep;
    }
  }
  
  return( stop );
}


//////
int
GsimResults()
{
#if DEBUG
    std::cerr << "GsimHits" << std::endl;
#endif
  
  int  nhit = 0;
  
  // CBAR
  int  ct = 0;
  for( int i=0;i<32;i++ ){
    if ( nresult.ESCI00[i] + nresult.ESUS[i] + 
         nresult.ESCI[i] + nresult.ELEAD[i] > 0 ){
      nresult.ESCI00[ct]  = nresult.ESCI00[i];
      nresult.ESUS[ct]    = nresult.ESUS[i];
      nresult.ESCI[ct]    = nresult.ESCI[i];
      nresult.ELEAD[ct]   = nresult.ELEAD[i];
      nresult.IDCBR[ct]   = i;
      ct++;
    }
  }
  nresult.NCBR = ct;
  nhit += ct;
  
  // copy GEN_PARTICLE to NRESULT table
  nresult.NGEN = gen_particle.NGEN;
  for( int i=0; i<gen_particle.NGEN; i++ ){
    Hep3Vector  pv( gen_particle.XX[i],gen_particle.YY[i],gen_particle.ZZ[i] );
    nresult.XX[i]      = gen_particle.XX[i];
    nresult.YY[i]      = gen_particle.YY[i];
    nresult.ZZ[i]      = gen_particle.ZZ[i];
    nresult.PX[i]      = gen_particle.PX[i];
    nresult.PY[i]      = gen_particle.PY[i];
    nresult.PZ[i]      = gen_particle.PZ[i];
    nresult.PP[i]      = gen_particle.PP[i];
    nresult.PTH[i]     = pv.theta();
    nresult.PPH[i]     = pv.phi();
    nresult.EK[i]      = gen_particle.EK[i];
    nresult.MASS[i]    = gen_particle.MASS[i];
    nresult.PN[i]      = gen_particle.PN[i];
    nresult.MO[i]      = gen_particle.MO[i];
    nresult.DA1[i]     = gen_particle.DA1[i];
    nresult.DA2[i]     = gen_particle.DA2[i];
    nresult.MECH[i]    = gen_particle.MECH[i];
    nresult.GSTATUS[i] = gen_particle.STATUS[i];
  }

  //
  return( nhit );
}

