// -*- C++ -*-
//
// GsimGeom_CentralBarrel.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/GsimGeom.h"
#include "GsimUserGeom.h"


//////
#define  DEBUG  0


//////
// FFkey
int    DetUseCBAR[] = { 1, 1 };
int    DetUseCS00[] = { 1, 1 };
float  DetCS00[]    = {  2, 0.5 };
float  DetCSCI[]    = { 45, 0.5 };
float  DetCPB1[]    = { 44, 0.2 };
float  DetCPB2[]    = {  0, 0.2 };


//////
void
GsimUserGeom_CentralBarrel()
{
  //
  int  ivol=0;

  // Central Barrel
  //double  cbar_height   = 42.356;
  double  cbar_height   = 45.00;
  double  cbar_len_bot  = 550;
  double  cbar_len_top  = 530;
  double  cbar_bottom   = 19.75;//100.3*tan(2*M_PI/64)*2;//19.51;
  double  cbar_top      = cbar_bottom + cbar_height*tan( 2*M_PI/32 );
  double  cbar_rmin     = 100.3;
  double  cbar_trap_th  = atan( (cbar_top-cbar_bottom)/2/cbar_height );
  double  cbar_trap_thd = rad2deg( cbar_trap_th );
  float  p_cbar[] = { cbar_height/2,cbar_trap_thd,0,
                      cbar_len_top/2, cbar_top/2,   cbar_top/2,    0,
                      cbar_len_bot/2, cbar_bottom/2,cbar_bottom/2, 0  };
  GSVOLU( "CBAR","TRAP",n_vacuum,p_cbar,sizeof(p_cbar)/sizeof(float),ivol );

  // SUS,Scintillator, Pb
  double  Tsus   = 0.3;
  double  Tgap   = 0.018+0.005; // 180 um sheet + 50 um space
  int     Nsci00 = (int)DetCS00[0];
  double  Tsci00 = DetCS00[1];
  int     Nsci   = (int)DetCSCI[0];
  double  Tsci   = DetCSCI[1];
  int     Nlead1 = (int)DetCPB1[0];
  double  Tlead1 = DetCPB1[1];
  int     Nlead2 = (int)DetCPB2[0];
  double  Tlead2 = DetCPB2[1];

  // 
  double  Dx = (cbar_top - cbar_bottom) / cbar_height;
  double  Dy = (cbar_len_top - cbar_len_bot) / cbar_height;

  // Scintilator00 ; first two layers
  GSVOLU( "CS00","TRAP",n_scintillator,0,0,ivol );
  if ( DetUseCBAR[1] && ////// use shower
       DetUseCS00[0]  ){
    for( int n=0;n<2;n++ ){
      double  dsci00 = (Tsci00+Tgap*2) * n;
      
      double  cs00_height   = Tsci00;
      double  cs00_len_bot  = cbar_len_bot + Dy * (dsci00+Tgap);
      double  cs00_len_top  = cs00_len_bot + Dy * cs00_height;
      double  cs00_bottom   = cbar_bottom + Dx * (dsci00+Tgap);
      double  cs00_top      = cs00_bottom + Dx * cs00_height;
      double  cs00_trap_th  = atan( (cs00_top-cs00_bottom)/2/cs00_height );
      double  cs00_trap_thd = rad2deg( cs00_trap_th );
      float  p_cs00[] = { cs00_height/2,cs00_trap_thd,0,
                          cs00_len_top/2, cs00_top/2,   cs00_top/2,    0,
                          cs00_len_bot/2, cs00_bottom/2,cs00_bottom/2, 0  };
      double  z = cbar_height/2 - dsci00 - Tgap - cs00_height/2;
      double  y = 0;
      double  x = Dx/2 * z;
      GSPOSP( "CS00",n+1,"CBAR",x,y,z,0,"ONLY",
              p_cs00,sizeof(p_cs00)/sizeof(float) );
    }
  }
  
  // SUS
  double  dsci00 = (Tsci00+Tgap*2) * 2;
  double  csus_height   = Tsus;
  double  csus_len_bot  = cbar_len_bot + Dy * dsci00;
  double  csus_len_top  = csus_len_bot + Dy * csus_height;
  double  csus_bottom   = cbar_bottom + Dx * dsci00;
  double  csus_top      = csus_bottom + Dx * csus_height;
  double  csus_trap_th  = atan( (csus_top-csus_bottom)/2/csus_height );
  double  csus_trap_thd = rad2deg( csus_trap_th );
  float  p_csus[] = { csus_height/2,csus_trap_thd,0,
                      csus_len_top/2, csus_top/2,   csus_top/2,    0,
                      csus_len_bot/2, csus_bottom/2,csus_bottom/2, 0  };
  GSVOLU( "CSUS","TRAP",n_iron,p_csus,sizeof(p_csus)/sizeof(float),ivol );
  if ( DetUseCBAR[1] ){ ////// use shower
    double  z = cbar_height/2 - dsci00 - csus_height/2;
    double  y = 0;
    double  x = Dx/2 * z;
    GSPOS( "CSUS",1,"CBAR",x,y,z,0,"ONLY" );
  }
  
  // Scintillator
  GSVOLU( "CSCI","TRAP",n_scintillator,0,0,ivol );
  for( int n=0; n<Nsci; n++ ){
    if ( DetUseCBAR[1] ){ ////// use shower
      double  dsci00 = (Tsci00+Tgap*2) * 2;
      double  dsci   = (Tsci+Tgap*2) * n;
      double  dlead  = Tlead1 * ((n<=Nlead1) ? n : Nlead1) 
                       + ((n>Nlead1) ? Tlead2*(n-Nlead1) : 0);
      double  dtotal = dsci00 + Tsus + dsci + dlead;

      double  csci_height   = Tsci;
      double  csci_len_bot  = cbar_len_bot + Dy * (dtotal + Tgap);
      double  csci_len_top  = csci_len_bot + Dy*csci_height;
      double  csci_bottom   = cbar_bottom + Dx * (dtotal + Tgap);
      double  csci_top      = csci_bottom + Dx*csci_height;
      double  csci_trap_th  = atan( (csci_top-csci_bottom)/2/csci_height );
      double  csci_trap_thd = rad2deg( csci_trap_th );
      float  p_csci[] = { csci_height/2,csci_trap_thd,0,
                          csci_len_top/2, csci_top/2,   csci_top/2,    0,
                          csci_len_bot/2, csci_bottom/2,csci_bottom/2, 0  };
      double  z = cbar_height/2 - dtotal - Tgap - Tsci/2;
      double  y = 0;
      double  x = Dx/2 * z;
      GSPOSP( "CSCI",n+1,"CBAR",x,y,z,0,"ONLY",
              p_csci,sizeof(p_csci)/sizeof(float) );
    }
  }

  // Pb
  GSVOLU( "CPBL","TRAP",n_lead,0,0,ivol );
  if ( DetUseCBAR[1] ){ ////// use shower
    for( int n=0; n<Nlead1+Nlead2; n++ ){
      double  dsci00 = (Tsci00+Tgap*2) * 2;
      double  dsci   = (Tsci+Tgap*2) * (n+1);
      double  dlead  = Tlead1 * ((n<=Nlead1) ? n : Nlead1) 
                       + ((n>Nlead1) ? Tlead2*(n-Nlead1) : 0);
      double  dtotal = dsci00 + Tsus + dsci + dlead;

      double  clead_height   = (n<Nlead1) ? Tlead1 : Tlead2;
      double  clead_len_bot  = cbar_len_bot + Dy * dtotal;
      double  clead_len_top  = clead_len_bot + Dy * clead_height;
      double  clead_bottom   = cbar_bottom + Dx * dtotal;
      double  clead_top      = clead_bottom + Dx * clead_height;
      double  clead_trap_th  = atan( (clead_top-clead_bottom)/2/clead_height );
      double  clead_trap_thd = rad2deg( clead_trap_th );
      float  p_clead[] = { clead_height/2,clead_trap_thd,0,
                           clead_len_top/2, clead_top/2,   clead_top/2,    0,
                           clead_len_bot/2, clead_bottom/2,clead_bottom/2, 0};
      double  z = cbar_height/2 - dtotal - clead_height/2;
      double  y = 0;
      double  x = Dx/2 * z;
      GSPOSP( "CPBL",n+1,"CBAR",x,y,z,0,"ONLY",
              p_clead,sizeof(p_clead)/sizeof(float) );
    }
  }

  // Install Module
  for( int i=0;i<32;i++ ){
    if ( i<3 || i>29 ){
      double  x = cbar_rmin + cbar_height/2;
      double  y = -cbar_bottom/2 + (cbar_top+cbar_bottom)/4;
      double  z = 135 + cbar_len_bot/2;
      Hep3Vector c( x,y,z );
      c.rotateZ( 2*M_PI*i/32 );
      int  rotm = grot_x90_z_32+(24+i)%32;
      if ( DetUseCBAR[0] ){ //////
        GSPOS( "CBAR",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
      }
    }
  }



}
