// -*- C++ -*-
//
// GsimUserGeom.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/GsimGeom.h"
#include "GsimUserGeom.h"


//////
#define  DEBUG  0


//////
// FFkey
int  DetUseFBAR[] = { 1, 1 };
int  DetUseCBAR[] = { 1, 1 };
int  DetUseCSIC[] = { 1, 1 };
int  DetUseCSEG[] = { 1, 1 };
int  DetUseCC01[] = { 1, 1 };
int  DetUseCC02[] = { 1, 1 };
int  DetUseCC03[] = { 1, 1 };
int  DetUseCC04[] = { 1, 1 };
int  DetUseCC05[] = { 1, 1 };
int  DetUseBMCT[] = { 1, 1 };
int  DetUseCHGV[] = { 1, 1 };


//////
void
GsimRotationMatrix()
{
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_z_32+i, 90, i*360./32,  90, i*360./32+90,  0, 0 );
  }
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_x90_z_32+i, 90, i*360./32,  0, 0,  90, -90+i*360./32 );
  }
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_y90_z_32+i, 180, 0,  90, 90+i*360./32,  90, i*360./32 );
  }
  
}


//////
void
GsimUserGeom()
{
  if ( gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "GsimUserGeom" << std::endl; 

  //
  GsimRotationMatrix();

  //
  int  ivol=0;

  // Mother volume
  float p_moth[] = {400./2, 400./2, 2400./2 }; // x,y,z
  GSVOLU( "MOTH","BOX ",n_vacuum,p_moth,sizeof(p_moth)/sizeof(float),ivol );

  // Front Barrel
  double  fbar_height = 41.2984;
  double  fbar_len    = 275;
  double  fbar_rmin   = 30.74;//31.7;
  double  fba1_height = 37.8304;
  double  fba1_bottom = 12.2291;
  double  fba1_top    = fba1_bottom + fba1_height*tan( M_PI/8 );
  double  fba1_alpha  = atan( (fba1_top-fba1_bottom)/2/fba1_height );
  double  fba1_alpdeg = rad2deg( fba1_alpha );
  float  p_fba1[] = { fbar_len/2, 0, 0, 
                      fba1_height/2,fba1_bottom/2,fba1_top/2,fba1_alpdeg,
                      fba1_height/2,fba1_bottom/2,fba1_top/2,fba1_alpdeg };
  GSVOLU( "FBA1","TRAP",n_lead,p_fba1,sizeof(p_fba1)/sizeof(float),ivol );
  for( int i=0;i<16;i++ ){
    double  x = fbar_rmin + fba1_height/2;
    double  y = fba1_bottom/2 -(fba1_top+fba1_bottom)/4;
    double  z = fbar_len/2;
    Hep3Vector c( x,y,z );
    c.rotateZ( 2*M_PI*i/16 );
    int  rotm = grot_z_32+(24+i*2)%32;
    if ( DetUseFBAR[0] ){ //////
      GSPOS( "FBA1",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
    }
  }
  // 
  double  fba2_height = fbar_height - fba1_height;
  double  fba2_bottom = fba1_top;
  double  fba2_top    = 14.05;
  double  fba2_alpha  = atan( (fba2_top-fba2_bottom)/2/fba2_height );
  double  fba2_alpdeg = rad2deg( fba2_alpha );
  float  p_fba2[] = { fbar_len/2, 0, 0, 
                      fba2_height/2,fba2_bottom/2,fba2_top/2,fba2_alpdeg,
                      fba2_height/2,fba2_bottom/2,fba2_top/2,fba2_alpdeg };
  GSVOLU( "FBA2","TRAP",n_lead,p_fba2,sizeof(p_fba2)/sizeof(float),ivol );
  for( int i=0;i<16;i++ ){
    double  x = fbar_rmin + fbar_height - fba2_height/2;
    double  y = fba1_bottom/2 -(fba2_top+fba2_bottom)/4;
    double  z = fbar_len/2;
    Hep3Vector c( x,y,z );
    c.rotateZ( 2*M_PI*i/16 );
    int  rotm = grot_z_32+(24+i*2)%32;
    if ( DetUseFBAR[0] ){ //////
      GSPOS( "FBA2",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
    }
  }
  // Front Barrel Backbone
  double  fbab_rmin = fbar_rmin + 41.3; // > fbar_rmin + fbar_height
  double  fbab_rmax = fbab_rmin + 2.2;
  float  p_fbab[] = { -360./16/2, 360, 16, 2,
                      -fbar_len/2, fbab_rmin, fbab_rmax,
                      +fbar_len/2, fbab_rmin, fbab_rmax };
  GSVOLU( "FBAB","PGON",n_iron,p_fbab,sizeof(p_fbab)/sizeof(float),ivol );
  double c_fbab_z = fbar_len/2;
  if ( DetUseFBAR[0] ){ //////
    GSPOS( "FBAB", 1, "MOTH", 0,0,c_fbab_z, 0, "ONLY" );
  }

  // Central Barrel
  double  cbar_height   = 42.356;
  double  cbar_len_bot  = 550;
  double  cbar_len_top  = 530;
  double  cbar_bottom   = 19.51;
  double  cbar_top      = cbar_bottom + cbar_height*tan( 2*M_PI/32 );
  double  cbar_rmin     = 100.3;
  double  cbar_trap_th  = atan( (cbar_top-cbar_bottom)/2/cbar_height );
  double  cbar_trap_thd = rad2deg( cbar_trap_th );
  float  p_cbar[] = { cbar_height/2,cbar_trap_thd,0,
                      cbar_len_top/2, cbar_top/2,   cbar_top/2,    0,
                      cbar_len_bot/2, cbar_bottom/2,cbar_bottom/2, 0  };
  GSVOLU( "CBAR","TRAP",n_lead,p_cbar,sizeof(p_cbar)/sizeof(float),ivol );
  for( int i=0;i<32;i++ ){
    double  x = cbar_rmin + cbar_height/2;
    double  y = -cbar_bottom/2 + (cbar_top+cbar_bottom)/4;
    double  z = 135 + cbar_len_bot/2;
    Hep3Vector c( x,y,z );
    c.rotateZ( 2*M_PI*i/32 );
    int  rotm = grot_x90_z_32+(24+i)%32;
    if ( DetUseCBAR[0] ){ //////
      GSPOS( "CBAR",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
    }
  }

  // CsI crystal
  double  csic_len  = 30;    // CsI crystal size
  double  csic_xy   = 7;     // 
  double  csic_gap  = 0.015; // reflector sheet
  float  p_csic[] = { csic_xy/2, csic_xy/2, csic_len/2 };
  GSVOLU( "CSIC","BOX ",n_csi,p_csic,sizeof(p_csic)/sizeof(float),ivol );
  // deformed CsI crystals
  GSVOLU( "CSDB","BOX ",n_csi,0,0,ivol );
  GSVOLU( "CSDT","TRAP",n_csi,0,0,ivol );
  double csdt_top[7],csdt_bot[7],csdt_hi[7],csdb_y[7];
  // #1
  csdt_top[1] = 5.1228;
  csdt_bot[1] = 7;
  csdt_hi[1]  = 0.1752;
  csdb_y[1]   = 6.8248;
  // #2
  csdt_top[2] = 5.7604;
  csdt_bot[2] = 6.8098;
  csdt_hi[2]  = 7;
  // #3
  csdt_top[3] = 4.1639;
  csdt_bot[3] = 5.7604;
  csdt_hi[3]  = 7;
  // #4
  csdt_top[4] = 5.2141;
  csdt_bot[4] = 7;
  csdt_hi[4]  = 0.7560;
  csdb_y[4]   = 6.2440;
  // #5
  csdt_top[5] = 4.7346;
  csdt_bot[5] = 7;
  csdt_hi[5]  = 1.4439;
  csdb_y[5]   = 5.5561;
  // #6
  csdt_top[6] = 0.4929;
  csdt_bot[6] = 7;
  csdt_hi[6]  = 5.8158;
  csdb_y[6]   = 1.1842;

  // CsI region
  double  csir_len  = 80;    // CsI region
  double  csir_rmin = 0;
  double  csir_rmax = 97.5;
  float  p_csir[] = { csir_rmin,csir_rmax, csir_len/2 };
  GSVOLU( "CSI ","TUBE",n_vacuum,p_csir,sizeof(p_csir)/sizeof(float),ivol );
  double c_csir_x = 0;
  double c_csir_y = 0;
  double c_csir_z = 610 - 5 + csir_len/2;
  GSPOS( "CSI ", 1, "MOTH", c_csir_x,c_csir_y,c_csir_z, 0, "ONLY" );

  // CsI Cylinder
  double  cscy_thick = 2.0; // 20 mm thick SUS
  double  cscy_len  = csir_len - 5;
  double  cscy_rmax = csir_rmax;
  double  cscy_rmin = cscy_rmax - cscy_thick;
  float  p_cscy[] = { cscy_rmin,cscy_rmax, cscy_len/2 };
  GSVOLU( "CSCY","TUBE",n_iron,p_cscy,sizeof(p_cscy)/sizeof(float),ivol );
  GSPOS( "CSCY", 1, "CSI ", 0,0,2.5, 0, "ONLY" );

  // CsI Dummy Inner Compartment 1,2
  double  csh1_len = csir_len;
  double  csh1_xyo = (csic_xy+csic_gap*2)*13;
  double  csh1_xyi = (csic_xy+csic_gap*2)*5;
  float  p_csh1[] = { -360./4/2, 360, 4, 2,
                      -csh1_len/2, csh1_xyi/2, csh1_xyo/2,
                      +csh1_len/2, csh1_xyi/2, csh1_xyo/2 };
  GSVOLU( "CSH1","PGON",n_vacuum,p_csh1,sizeof(p_csh1)/sizeof(float),ivol );
  GSPOS( "CSH1", 1, "CSI ", 0,0,0, 0, "ONLY" );

  double  csh2_len = csir_len;
  double  csh2_xyo = (csic_xy+csic_gap*2)*19;
  double  csh2_xyi = (csic_xy+csic_gap*2)*13;
  float  p_csh2[] = { -360./4/2, 360, 4, 2,
                      -csh2_len/2, csh2_xyi/2, csh2_xyo/2,
                      +csh2_len/2, csh2_xyi/2, csh2_xyo/2 };
  GSVOLU( "CSH2","PGON",n_vacuum,p_csh2,sizeof(p_csh2)/sizeof(float),ivol );
  GSPOS( "CSH2", 1, "CSI ", 0,0,0, 0, "ONLY" );
  
  // Small Edge Calorimator
  //if ( DetUseCSEG[0] ) //////
  GSVOLU( "CSEG","TRAP",n_lead,0,0,ivol );
  double  csvt_bot[3],csvt_hi[3];
  // #a
  csvt_bot[0] = 4.1939;
  csvt_hi[0]  = 12.2741;
  // #b
  csvt_bot[1] = 6.2590;
  csvt_hi[1]  = 11.7946;
  // #c
  csvt_bot[2] = 5.5711;
  csvt_hi[2]  = 7.5529;

  // Locate CsI Crystal
  char  csiblock[][28] = {
    // 0         1         2      
    { ".........a3210123a........." },  //  0
    { ".......b40000000004b......." },  //  1
    { ".....c500000000000005c....." },  //  2
    { "....6000000000000000006...." },  //  3
    { "...600000000000000000006..." },  //  4
    { "..c000000000000000000000c.." },  //  5
    { "..50000000000000000000005.." },  //  6
    { ".b00000000000000000000000b." },  //  7
    { ".4000000000000000000000004." },  //  8
    { "a0000000000000000000000000a" },  //  9
    { "300000000000000000000000003" },  // 10
    { "20000000000.....00000000002" },  // 11
    { "10000000000.....00000000001" },  // 12
    { "00000000000.....00000000000" } };// 13
  
  int  id = 0;
  for( int y=0;y<27;y++ ){
    for( int x=0;x<27;x++ ){
      int  yy = (y<13) ? y : 26-y;
      double c_csic_x = (x-13)*(csic_xy+csic_gap*2);
      double c_csic_y = (y-13)*(csic_xy+csic_gap*2);
      double c_csic_z = -csir_len/2 + 5 + csic_len/2;
      if ( csiblock[yy][x] == '0' ){
        id++;
        if ( DEBUG )
          std::cout <<"CsI(" << id << ")=(" <<  x << "," << y 
                    << ") : " << csiblock[yy][x]
                    << std::endl;
        if ( DetUseCSIC[0] ){ //////
          if ( 13-6<=x && x<=13+6 && 13-6<=y && y<=13+6 )
            GSPOS( "CSIC",id,"CSH1",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
          else if ( 13-9<=x && x<=13+9 && 13-9<=y && y<=13+9 )
            GSPOS( "CSIC",id,"CSH2",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
          else
            GSPOS( "CSIC",id,"CSI ",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
        }
      }
      else if ( '0' < csiblock[yy][x] && csiblock[yy][x] <= '9'  ){
        int  cd = csiblock[yy][x] - '0';
        id++;
        if ( DEBUG )
          std::cout <<"CsI(" << id << ")=(" <<  x << "," << y 
                    << ") : " << csiblock[yy][x]
                    << std::endl;
        if ( DetUseCSIC[0] ){ //////
          int  sec = (int)( fmod( Hep3Vector(x-13,y-13,0).phi()+2*M_PI,2*M_PI)
                            / (M_PI/4) );
          // 
          if ( !(cd == 2 || cd == 3) ){
            double  cbx = c_csic_x;
            double  cby = c_csic_y;
            double  cbz = c_csic_z;
            double  offset = csic_xy/2-csdb_y[cd]/2;
            int  rotb;
            if ( sec % 2 == 0 ){
              rotb = grot_z_32 + (24+sec*4) % 32;
              if ( sec == 0 ) cbx -= offset;
              if ( sec == 2 ) cby -= offset;
              if ( sec == 4 ) cbx += offset;
              if ( sec == 6 ) cby += offset;
            }
            else{
              rotb = grot_z_32 + (sec-1)*4;
              if ( sec == 1 ) cby -= offset;
              if ( sec == 3 ) cbx += offset;
              if ( sec == 5 ) cby += offset;
              if ( sec == 7 ) cbx -= offset;
            }
            float  p_csdb[] = { csic_xy/2,csdb_y[cd]/2,csic_len/2 };
            GSPOSP( "CSDB",id,"CSI ",cbx,cby,cbz,rotb,"ONLY",
                    p_csdb,sizeof(p_csdb)/sizeof(float) );
          }
          //
          double  ctx = c_csic_x;
          double  cty = c_csic_y;
          double  ctz = c_csic_z;
          double  offset   = csic_xy/2-(csdt_top[cd]+csdt_bot[cd])/4;
          double  offset_h = csic_xy/2-csdt_hi[cd]/2;
          int    rott;
          float  alpha = rad2deg( 
                         atan( (csdt_top[cd]-csdt_bot[cd])/2/csdt_hi[cd] ) );
          if ( cd == 2 || cd == 3 ){
            rott = ( sec%2 ) ? grot_z_32+(24+(sec-1)*4)%32 : grot_z_32+sec*4;
            alpha = ( sec%2 ) ? -alpha : alpha;
            if ( sec == 0 ) ctx -= offset;
            if ( sec == 1 ) cty -= offset;
            if ( sec == 2 ) cty -= offset;
            if ( sec == 3 ) ctx += offset;
            if ( sec == 4 ) ctx += offset;
            if ( sec == 5 ) cty += offset;
            if ( sec == 6 ) cty += offset;
            if ( sec == 7 ) ctx -= offset;
          }
          else {
            rott = ( sec%2 ) ?  grot_z_32+(sec-1)*4 : grot_z_32+(24+sec*4)%32;
            alpha = ( sec%2 ) ? alpha : -alpha;
            if ( sec == 0 ){ ctx += offset_h; cty -= offset;   }
            if ( sec == 1 ){ ctx -= offset;   cty += offset_h; }
            if ( sec == 2 ){ ctx += offset;   cty += offset_h; }
            if ( sec == 3 ){ ctx -= offset_h; cty -= offset;   }
            if ( sec == 4 ){ ctx -= offset_h; cty += offset;   }
            if ( sec == 5 ){ ctx += offset;   cty -= offset_h; }
            if ( sec == 6 ){ ctx -= offset;   cty -= offset_h; }
            if ( sec == 7 ){ ctx += offset_h; cty += offset;   }
          }
          float p_csdt[]={ csic_len/2, 0, 0,
                           csdt_hi[cd]/2,csdt_bot[cd]/2,csdt_top[cd]/2,alpha,
                           csdt_hi[cd]/2,csdt_bot[cd]/2,csdt_top[cd]/2,alpha };
          GSPOSP( "CSDT",id,"CSI ",ctx,cty,ctz,rott,"ONLY",
                  p_csdt,sizeof(p_csdt)/sizeof(float) );
        }
      }
      else if ( 'a' <= csiblock[yy][x] && csiblock[yy][x] <= 'z'  ){
        id++;
        int  cd = csiblock[yy][x] - 'a';
        int  sec = (int)( fmod( Hep3Vector(x-13,y-13,0).phi()+2*M_PI,2*M_PI)
                          / (M_PI/4) );
        double  ctx = c_csic_x;
        double  cty = c_csic_y;
        double  ctz = c_csic_z;
        double  offset   = csic_xy/2 - csvt_bot[cd]/4;
        double  offset_h = csic_xy/2 - csvt_hi[cd]/2;
        if ( sec == 0 ){ ctx -= offset;   cty -= offset_h; }
        if ( sec == 1 ){ ctx -= offset_h; cty -= offset;   }
        if ( sec == 2 ){ ctx += offset_h; cty -= offset;   }
        if ( sec == 3 ){ ctx += offset;   cty -= offset_h; }
        if ( sec == 4 ){ ctx += offset;   cty += offset_h; }
        if ( sec == 5 ){ ctx += offset_h; cty += offset;   }
        if ( sec == 6 ){ ctx -= offset_h; cty += offset;   }
        if ( sec == 7 ){ ctx -= offset;   cty += offset_h; }
        float  alpha = rad2deg( atan( csvt_bot[cd]/2/csvt_hi[cd] ) );
        alpha = ( sec%2 ) ? alpha : -alpha;
        int  rott = (sec%2) ? grot_z_32+(24+(sec-1)*4)%32 : grot_z_32+sec*4;
        float  p_csvt[]={ csic_len/2, 0, 0,
                          csvt_hi[cd]/2,csvt_bot[cd]/2, 0, alpha,
                          csvt_hi[cd]/2,csvt_bot[cd]/2, 0, alpha };
        if ( DetUseCSEG[0] ){ //////
          GSPOSP( "CSEG",id,"CSI ",ctx,cty,ctz,rott,"ONLY",
                  p_csvt,sizeof(p_csvt)/sizeof(float) );
        }
      }
    }
  }
  
  // CC01
  double  cc01_len  = 25;
  double  cc01_rmin = 4.3;
  double  cc01_rmax = 31;
  float  p_cc01[] = { cc01_rmin,cc01_rmax, cc01_len/2 };
  GSVOLU( "CC01","TUBE",n_lead,p_cc01,sizeof(p_cc01)/sizeof(float),ivol );
  double c_cc01_z = cc01_len/2;
  if ( DetUseCC01[0] ){ //////
    GSPOS( "CC01", 1, "MOTH", 0,0,c_cc01_z, 0, "ONLY" );
  }

  // CC02
  double  cc02_len  = 25;
  double  cc02_rmin = 6.2;
  double  cc02_rmax = 31;
  float  p_cc02[] = { cc02_rmin,cc02_rmax, cc02_len/2 };
  GSVOLU( "CC02","TUBE",n_lead,p_cc02,sizeof(p_cc02)/sizeof(float),ivol );
  double c_cc02_z = fbar_len - cc02_len/2;
  if ( DetUseCC02[0] ){ //////
    GSPOS( "CC02", 1, "MOTH", 0,0,c_cc02_z, 0, "ONLY" );
  }

  // CC03
  double  cc03_len = 40;
  double  cc03_xyo = (csic_xy+csic_gap*2)*5; // = 35.15 cm
  double  cc03_xyi = 11.2;
  float  p_cc03[] = { -360./4/2, 360, 4, 2,
                      -cc03_len/2, cc03_xyi/2, cc03_xyo/2,
                      +cc03_len/2, cc03_xyi/2, cc03_xyo/2 };
  GSVOLU( "CC03","PGON",n_vacuum,p_cc03,sizeof(p_cc03)/sizeof(float),ivol );
  double c_cc03_z = -csir_len/2 + cc03_len/2;
  if ( DetUseCC03[0] ){ //////
    GSPOS( "CC03", 1, "CSI ", 0,0,c_cc03_z, 0, "ONLY" );
  }

  // CC03 Inner Support SUS Frame
  double  cc3i_len = cc03_len;
  double  cc3i_xyo = 11.6; // = cc3m_x - cc3m_y
  double  cc3i_xyi = 11.2; // 2 mm thick x 2
  float  p_cc3i[] = { -360./4/2, 360, 4, 2,
                      -cc3i_len/2, cc3i_xyi/2, cc3i_xyo/2,
                      +cc3i_len/2, cc3i_xyi/2, cc3i_xyo/2 };
  GSVOLU( "CC3I","PGON",n_iron,p_cc3i,sizeof(p_cc3i)/sizeof(float),ivol );
  if ( DetUseCC03[0] ){ //////
    GSPOS( "CC3I", 1, "CC03", 0,0,0, 0, "ONLY" );
  }

  // CC03 module
  double  cc3w_thick = 0.1;  // tungsten
  double  cc3s_thick = 0.34; // scintillator
  double  cc3s_gap   = 0.00; // wrapping sheet of scintillator (no sheet?)
  double  cc3m_len = cc03_len;
  double  cc3m_x   = 22.94;
  double  cc3m_y   = 11.34; // 26*scinti + 25*tungsten
  float  p_cc3m[] = { cc3m_x/2, cc3m_y/2, cc3m_len/2 };
  GSVOLU( "CC3M","BOX ",n_vacuum,p_cc3m,sizeof(p_cc3m)/sizeof(float),ivol );
  double c_cc3m_x = (cc3m_x+cc3m_y)/2-cc3m_x/2;
  double c_cc3m_y = (cc3m_x+cc3m_y)/2-cc3m_y/2;
  GSPOS( "CC3M", 1, "CC03",  c_cc3m_y, c_cc3m_x,0, grot_z_32+32*3/4, "ONLY" );
  GSPOS( "CC3M", 2, "CC03", -c_cc3m_x, c_cc3m_y,0, 0,                "ONLY" );
  GSPOS( "CC3M", 3, "CC03", -c_cc3m_y,-c_cc3m_x,0, grot_z_32+32*1/4, "ONLY" );
  GSPOS( "CC3M", 4, "CC03",  c_cc3m_x,-c_cc3m_y,0, grot_z_32+32*2/4, "ONLY" );

  // CC03 components
  float  p_cc3s[] = { cc3m_x/2, cc3s_thick/2, cc3m_len/2 };
  GSVOLU( "CC*S","BOX ",n_scintillator,
          p_cc3s,sizeof(p_cc3s)/sizeof(float),ivol );
  for( int i=0;i<26;i++ ){
    double  dd = cc3w_thick + cc3s_gap*2 + cc3s_thick;
    double  cy = -cc3m_y/2 + dd*i + cc3s_gap + cc3s_thick/2;
    if ( DetUseCC03[0] ){ //////
      GSPOS( "CC*S", i+1, "CC3M", 0,cy,0, 0, "ONLY" );
    }
  }
  float  p_cc3w[] = { cc3m_x/2, cc3w_thick/2, cc3m_len/2 };
  GSVOLU( "CC*W","BOX ",n_tungsten,
          p_cc3w,sizeof(p_cc3w)/sizeof(float),ivol );
  for( int i=0;i<25;i++ ){
    double  dd = cc3w_thick + cc3s_gap*2 + cc3s_thick;
    double  cy = -cc3m_y/2 + dd*(i+1) - cc3w_thick/2;
    if ( DetUseCC03[0] ){ //////
      GSPOS( "CC*W", i+1, "CC3M", 0,cy,0, 0, "ONLY" );
    }
  }

  // CC04
  double  cc04_len  = 40;
  double  cc04_rmin = 6.5;
  double  cc04_rmax = 60./2;
  float  p_cc04[] = { -360./4/2, 360, 4, 2,
                      -cc04_len/2,cc04_rmin,cc04_rmax,
                      +cc04_len/2,cc04_rmin,cc04_rmax  };
  GSVOLU( "CC04","PGON",n_lead,
          p_cc04,sizeof(p_cc04)/sizeof(float),ivol );
  double c_cc04_z = 700 + cc04_len/2;
  if ( DetUseCC04[0] ){ //////
    GSPOS( "CC04", 1, "MOTH", 0,0,c_cc04_z, 0, "ONLY" );
  }

  // CC05
  double  cc05_len  = 40;
  double  cc05_rmin = 7.0;
  double  cc05_rmax = 60./2;
  float  p_cc05[] = { -360./4/2, 360, 4, 2,
                      -cc05_len/2,cc05_rmin,cc05_rmax,
                      +cc05_len/2,cc05_rmin,cc05_rmax  };
  GSVOLU( "CC05","PGON",n_lead,
          p_cc05,sizeof(p_cc05)/sizeof(float),ivol );
  double c_cc05_z = 850 + cc05_len/2;
  if ( DetUseCC05[0] ){ //////
    GSPOS( "CC05", 1, "MOTH", 0,0,c_cc05_z, 0, "ONLY" );
  }

  // Beam Catcher
  double  bmct_len  = 20;
  double  bmct_rmin = 0;
  double  bmct_rmax = 15./2;
  float  p_bmct[] = { bmct_rmax, bmct_rmax, bmct_len/2 };
  GSVOLU( "BMCT","BOX ",n_scintillator,
          p_bmct,sizeof(p_bmct)/sizeof(float),ivol );
  double c_bmct_z = 895 + bmct_len/2;
  if ( DetUseBMCT[0] ){ //////
    GSPOS( "BMCT", 1, "MOTH", 0,0,c_bmct_z, 0, "ONLY" );
  }

  // Charged Veto
  // inner part
  int     chv_ndiv  = 32;
  double  chv_thick = 0.6; // 6 mm scintillator
  double  chv1_len  = 40 - chv_thick;
  double  chv1_rmin = 8;
  double  chv1_rmax = chv1_rmin + chv_thick;
  float  p_chv1[] = { -360./32/2, 360./32, 1, 2,
                      -chv1_len/2, chv1_rmin, chv1_rmax,
                      +chv1_len/2 ,chv1_rmin, chv1_rmax };
  GSVOLU( "CHV1","PGON",n_scintillator,
          p_chv1,sizeof(p_chv1)/sizeof(float),ivol );
  double c_chv1_z = c_csir_z - csir_len/2 - chv1_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV1", i+1, "MOTH", 0,0,c_chv1_z, grot_z_32+i, "ONLY" );
    }
  }
  // plate
  double  chv2_len  = chv_thick;
  double  chv2_rmin = chv1_rmin;
  double  chv2_rmax = 99 - 45;
  float  p_chv2[] = { -360./32/2, 360./32, 1, 2,
                      -chv2_len/2, chv2_rmin, chv2_rmax,
                      +chv2_len/2 ,chv2_rmin, chv2_rmax };
  GSVOLU( "CHV2","PGON",n_scintillator,
          p_chv2,sizeof(p_chv2)/sizeof(float),ivol );
  double c_chv2_z = c_chv1_z - chv1_len/2 - chv2_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV2", i+1, "MOTH", 0,0,c_chv2_z, grot_z_32+i, "ONLY" );
    }
  }
  // bending part
  double  chv3_rmax  = 45; // !!!
  double  chv3_rmin  = chv3_rmax - chv_thick;
  double  chv3_len   = chv2_rmax*tan(M_PI/chv_ndiv)*2 +0.0; // 0.0cm overlap
  double  chv3_phmin = 0;
  double  chv3_phmax = 90;
  Hep3Vector  chv3_z1(0,0,-1),chv3_z2(0,0,1);
  chv3_z1.rotateX(-M_PI/chv_ndiv);
  chv3_z2.rotateX( M_PI/chv_ndiv);
  float  p_chv3[] = { chv3_rmin,chv3_rmax,chv3_len/2,
                      chv3_phmin,chv3_phmax,
                      chv3_z1.x(),chv3_z1.y(),chv3_z1.z(),
                      chv3_z2.x(),chv3_z2.y(),chv3_z2.z()  };
  GSVOLU( "CHV3","CTUB",n_scintillator,
          p_chv3,sizeof(p_chv3)/sizeof(float),ivol );
  for( int i=0;i<chv_ndiv;i++ ){
    Hep3Vector  c_chv3( chv2_rmax, 0, c_chv1_z+chv1_len/2+5 );
    c_chv3.rotateZ( 2*M_PI/chv_ndiv*i );
    int  rotm = grot_y90_z_32+(24+i*(32/chv_ndiv))%32;
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV3",i+1,"MOTH", c_chv3.x(),c_chv3.y(),c_chv3.z(),
             rotm, "ONLY" );
    }
  }
  // outer part
  double  chv4_len  = 10;
  double  chv4_rmax = chv2_rmax + chv3_rmax;
  double  chv4_rmin = chv4_rmax - chv_thick;
  float  p_chv4[] = { -360./32/2, 360./32, 1, 2,
                      -chv4_len/2, chv4_rmin, chv4_rmax,
                      +chv4_len/2 ,chv4_rmin, chv4_rmax };
  GSVOLU( "CHV4","PGON",n_scintillator,
          p_chv4,sizeof(p_chv4)/sizeof(float),ivol );
  double c_chv4_z = c_csir_z - csir_len/2 + 5 + chv4_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV4", i+1, "MOTH", 0,0,c_chv4_z, grot_z_32+i, "ONLY" );
    }
  }
  //
}


