// -*- C++ -*-
//
// E391Detector.h
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#ifndef E391DETECTOR_H_INCLUDED
#define E391DETECTOR_H_INCLUDED

//
#include "midas.h"
#include "unpack/AdcEvent.h"
#include "unpack/TdcEvent.h"
#include "detparam/DetParam.h"
#include "detector/Detector.h"

// Number of channels
#define N_C6            2
#define N_FRONTBARREL  32
#define N_CC01          8
#define N_CC02          8
#define N_MAINBARREL  192
#define N_CHARGEVETO   32
#define N_CSI         576  // normal 496 + deformed 56 + KTeV CsI 24 
#define N_SANDWICH     24
#define N_CC03          6
#define N_CC04          4
#define N_CC05          4
#define N_BEAMCATCHER  20

#define N_SUM_C6            1
#define N_SUM_FRONTBARREL   4
#define N_SUM_CC01          1
#define N_SUM_CC02          1
#define N_SUM_MAINBARREL   24
#define N_SUM_CHARGEVETO    4
#define N_SUM_CSI          72
#define N_SUM_SANDWICH      3
#define N_SUM_CC03          1
#define N_SUM_CC04          1
#define N_SUM_CC05          1
#define N_SUM_BEAMCATCHER   3

//
class E391Detector{
public:
  // singleton
  static E391Detector&  instance();

  // extractor
  inline const Detector&  C6( int id ) const;
  inline const Detector&  FrontBarrel( int id ) const;
  inline const Detector&  CC01( int id ) const;
  inline const Detector&  CC02( int id ) const;
  inline const Detector&  MainBarrel( int id ) const;
  inline const Detector&  ChargeVeto( int id ) const;
  inline const Detector&  CsI( int id ) const;
  inline const Detector&  Sandwich( int id ) const;
  inline const Detector&  CC03( int id ) const;
  inline const Detector&  CC04( int id ) const;
  inline const Detector&  CC05( int id ) const;
  inline const Detector&  BeamCatcher( int id ) const;
  
  inline const Detector&  C6_Sum( int id ) const;
  inline const Detector&  FrontBarrel_Sum( int id ) const;
  inline const Detector&  CC01_Sum( int id ) const;
  inline const Detector&  CC02_Sum( int id ) const;
  inline const Detector&  MainBarrel_Sum( int id ) const;
  inline const Detector&  ChargeVeto_Sum( int id ) const;
  inline const Detector&  CsI_Sum( int id ) const;
  inline const Detector&  Sandwich_Sum( int id ) const;
  inline const Detector&  CC03_Sum( int id ) const;
  inline const Detector&  CC04_Sum( int id ) const;
  inline const Detector&  CC05_Sum( int id ) const;
  inline const Detector&  BeamCatcher_Sum( int id ) const;
  
  inline const AdcEvent&  adcCrate( int crate ) const;
  inline const TdcEvent&  tdcCrate( int crate ) const;

  inline const AdcEvent*  pAdcCrate( int crate ) const;
  inline const TdcEvent*  pTdcCrate( int crate ) const;
  
  //
  int     runNumber() const { return m_runNumber; }
  int     spillNumber() const;
  int     eventNumber() const;

  void*   pbank( const std::string& bankname ) const;
  bool    exist( const std::string& bankname ) const;
  
  void    setRunNumber( int rn ) { m_runNumber = rn; }
  int     getDB( int run_number=0 );
  int     unpack( EVENT_HEADER* pheader, void* pevent ); 

 
protected:
  E391Detector();
  ~E391Detector();

private:
  // singleton
  static E391Detector*  s_e391det;

  // detparam
  DetParam*  m_detparam;
  
  // detector
  Detector  m_C6[ N_C6 ];
  Detector  m_FrontBarrel[ N_FRONTBARREL ];
  Detector  m_CC01[ N_CC01 ];
  Detector  m_CC02[ N_CC02 ];
  Detector  m_MainBarrel[ N_MAINBARREL ];
  Detector  m_ChargeVeto[ N_CHARGEVETO ];
  Detector  m_CsI[ N_CSI ];
  Detector  m_Sandwich[ N_SANDWICH ];
  Detector  m_CC03[ N_CC03 ];
  Detector  m_CC04[ N_CC04 ];
  Detector  m_CC05[ N_CC05 ];
  Detector  m_BeamCatcher[ N_BEAMCATCHER ];

  Detector  m_C6_Sum[ N_SUM_C6 ];
  Detector  m_FrontBarrel_Sum[ N_SUM_FRONTBARREL ];
  Detector  m_CC01_Sum[ N_SUM_CC01 ];
  Detector  m_CC02_Sum[ N_SUM_CC02 ];
  Detector  m_MainBarrel_Sum[ N_SUM_MAINBARREL ];
  Detector  m_ChargeVeto_Sum[ N_SUM_CHARGEVETO ];
  Detector  m_CsI_Sum[ N_SUM_CSI ];
  Detector  m_Sandwich_Sum[ N_SUM_SANDWICH ];
  Detector  m_CC03_Sum[ N_SUM_CC03 ];
  Detector  m_CC04_Sum[ N_SUM_CC04 ];
  Detector  m_CC05_Sum[ N_SUM_CC05 ];
  Detector  m_BeamCatcher_Sum[ N_SUM_BEAMCATCHER ];

  const static Detector  s_dummydet; // dummy
  const static AdcEvent  s_dummyAdcEvent; // dummy
  const static TdcEvent  s_dummyTdcEvent; // dummy

  // data
  AdcEvent*  m_adcCrate[MAX_NADCCRATE];
  TdcEvent*  m_tdcCrate[MAX_NTDCCRATE];

  int            m_runNumber;
  EVENT_HEADER*  m_pheader;
  void*          m_pevent;

  // method
  void    build();
};

// inline
#include "E391Detector/E391Detector.icc"


#endif // DETECTOR_H_INCLUDED

