// -*- C++ -*-
//
// E391Detector.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <stdio.h>
#include "midas.h"
#include "unpack/AdcEvent.h"
#include "unpack/TdcEvent.h"
#include "detparam/DetParam.h"
#include "E391Detector/E391Detector.h"

#define DEBUG  0


// static members
E391Detector*  E391Detector::s_e391det = 0;

const Detector  E391Detector::s_dummydet;
const AdcEvent  E391Detector::s_dummyAdcEvent;
const TdcEvent  E391Detector::s_dummyTdcEvent;


// singleton
E391Detector&
E391Detector::instance()
{
  //std::cout << "E391Detector::instance()" << std::endl;

  if ( s_e391det == 0 ){
    s_e391det = new E391Detector;
    if ( s_e391det == 0 ){ // failed
      std::cout << "E391Detector::instance() :"
                << " cannot create E391Detector object."
                << std::endl;
      perror( "new" );
      abort();
    }
  }

  return( *s_e391det );
}
  

// 
E391Detector::E391Detector()
  : m_detparam( 0 )
{
  if ( DEBUG )
    std::cout << "E391Detector::E391Detector()" << std::endl;
  //
  m_adcCrate[0] = m_adcCrate[1] = 0;
  m_tdcCrate[0] = m_tdcCrate[1] = 0;
}


//
E391Detector::~E391Detector()
{
  if ( DEBUG )
    std::cout << "E391Detector::~E391Detector()" << std::endl;

  delete m_tdcCrate[1];
  delete m_tdcCrate[0];
  delete m_adcCrate[1];
  delete m_adcCrate[0];
}


//
int
E391Detector::getDB( int runnum )
{
  if ( DEBUG )
    std::cout << "E391Detector::getDB()" << std::endl;
  
  // 
  m_detparam = &DetParam::instance();
  m_detparam->getDB( runnum );

  // Detector channels setup
  for( std::list<E391_DETE_DB>::const_iterator p=m_detparam->paramList().begin();
       p!=m_detparam->paramList().end(); p++ ){
 
    if ( p->name == "CSI" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CSI ){
        //std::cout << " CsI ID=" << p->ID << std::endl;
        m_CsI[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CV" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CHARGEVETO ){
        //std::cout << " CV   ID=" << p->ID << std::endl;
        m_ChargeVeto[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "SANDWICH" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_SANDWICH ){
        //std::cout << " SAND ID=" << p->ID << std::endl;
        m_Sandwich[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC03" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CC03 ){
        //std::cout << " CC03 ID=" << p->ID << std::endl;
        m_CC03[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC04" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CC04 ){
        //std::cout << " CC04 ID=" << p->ID << std::endl;
        m_CC04[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC05" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CC05 ){
        //std::cout << " CC05 ID=" << p->ID << std::endl;
        m_CC05[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "BC" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_BEAMCATCHER ){
        //std::cout << " BC   ID=" << p->ID << std::endl;
        m_BeamCatcher[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "C6" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_C6 ){
        //std::cout << " C6   ID=" << p->ID << std::endl;
        m_C6[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "FRONTBARREL" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_FRONTBARREL ){
        //std::cout << " FBAR ID=" << p->ID << std::endl;
        m_FrontBarrel[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC01" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CC01 ){
        //std::cout << " CC01 ID=" << p->ID << std::endl;
        m_CC01[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC02" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_CC02 ){
        //std::cout << " CC02 ID=" << p->ID << std::endl;
        m_CC02[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "MAINBARREL" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAINBARREL ){
        //std::cout << " MBAR ID=" << p->ID << std::endl;
        m_MainBarrel[ p->ID ].setDB( &(*p) );
      }
    }
    //
    else if ( p->name == "CSI" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CSI ){
        //std::cout << " CsI Sum ID=" << p->ID << std::endl;
        m_CsI_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CV" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CHARGEVETO ){
        //std::cout << " CV Sum   ID=" << p->ID << std::endl;
        m_ChargeVeto_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "SANDWICH" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_SANDWICH ){
        //std::cout << " SAND Sum ID=" << p->ID << std::endl;
        m_Sandwich_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC03" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CC03 ){
        //std::cout << " CC03 Sum ID=" << p->ID << std::endl;
        m_CC03_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC04" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CC04 ){
        //std::cout << " CC04 Sum ID=" << p->ID << std::endl;
        m_CC04_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC05" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CC05 ){
        //std::cout << " CC05 Sum ID=" << p->ID << std::endl;
        m_CC05_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "BC" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_BEAMCATCHER ){
        //std::cout << " BC Sum   ID=" << p->ID << std::endl;
        m_BeamCatcher_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "C6" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_C6 ){
        //std::cout << " C6 Sum   ID=" << p->ID << std::endl;
        m_C6_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "FRONTBARREL" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_FRONTBARREL ){
        //std::cout << " FBAR Sum ID=" << p->ID << std::endl;
        m_FrontBarrel_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC01" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CC01 ){
        //std::cout << " CC01 Sum ID=" << p->ID << std::endl;
        m_CC01_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "CC02" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_CC02 ){
        //std::cout << " CC02 Sum ID=" << p->ID << std::endl;
        m_CC02_Sum[ p->ID ].setDB( &(*p) );
      }
    }
    else if ( p->name == "MAINBARREL" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_SUM_MAINBARREL ){
        //std::cout << " MBAR Sum ID=" << p->ID << std::endl;
        m_MainBarrel_Sum[ p->ID ].setDB( &(*p) );
      }
    }

  }

  return( 0 );
}


//
int
E391Detector::unpack( EVENT_HEADER* pheader, void* pevent )
{
  if ( DEBUG )
    std::cout << "E391Detector::unpack()" << std::endl;
  //
  delete m_tdcCrate[1];
  delete m_tdcCrate[0];
  delete m_adcCrate[1];
  delete m_adcCrate[0];

  //
  m_pheader = pheader;
  m_pevent  = pevent;

  // banks
  DWORD*  pdataTDC0;
  int  nTDC0 = bk_locate( m_pevent, "TDC0", &pdataTDC0 );
  DWORD*  pdataTDC1;
  int  nTDC1 = bk_locate( m_pevent, "TDC1", &pdataTDC1 );

  DWORD*  pdataADC0;
  int  nADC0 = bk_locate( m_pevent, "ADC0", &pdataADC0 );
  DWORD*  pdataADC1;
  int  nADC1 = bk_locate( m_pevent, "ADC1", &pdataADC1 );

  DWORD*  pdataSMP0;
  int  ndataSMP0 = bk_locate( m_pevent, "SMP0", &pdataSMP0 );
  DWORD*  pdataSMP1;
  int  ndataSMP1 = bk_locate( m_pevent, "SMP1", &pdataSMP1 );

  DWORD*  pdataNGF0;
  int  ndataNGF0 = bk_locate( m_pevent, "NGF0", &pdataNGF0 );
  DWORD*  pdataNGF1;
  int  ndataNGF1 = bk_locate( m_pevent, "NGF1", &pdataNGF1 );

  DWORD*  pdataTRG0;
  int  ndataTRG0 = bk_locate( m_pevent, "TRG0", &pdataTRG0 );
  DWORD*  pdataTRG1;
  int  ndataTRG1 = bk_locate( m_pevent, "TRG1", &pdataTRG1 );

  DWORD*  pdataSCL0;
  int  ndataSCL0 = bk_locate( m_pevent, "SCL0", &pdataSCL0 );

  //
  m_tdcCrate[0] = new TdcEvent( 0, m_pheader, pdataTDC0, nTDC0, 1 );
  if ( m_tdcCrate[0]->serial() == 1 ){
    m_tdcCrate[0]->smp( pdataSMP0,ndataSMP0 ); // SMP
  }

  //
  m_tdcCrate[1] = new TdcEvent( 1, m_pheader, pdataTDC1, nTDC1, 1 );
  if ( m_tdcCrate[1]->serial() == 1 ){
    m_tdcCrate[1]->smp( pdataSMP1,ndataSMP1 ); // SMP
  }

  //
  m_adcCrate[0] = new AdcEvent( 0, m_pheader, pdataADC0, nADC0, 1 );
  if ( m_adcCrate[0]->serial() == 1 ){
    m_adcCrate[0]->ngf( pdataNGF0,ndataNGF0 );
    m_adcCrate[0]->trg( pdataTRG0,ndataTRG0 );
  }

  //
  m_adcCrate[1] = new AdcEvent( 1, m_pheader, pdataADC1, nADC1, 1 );
  if ( m_adcCrate[1]->serial() == 1 ){
    m_adcCrate[1]->ngf( pdataNGF1,ndataNGF1 );
    m_adcCrate[1]->trg( pdataTRG1,ndataTRG1 );
  }

  //
  return( 0 );
}


//
int
E391Detector::spillNumber() const
{
  const int*  pev = (int*)pbank( "EVNT" );
  if ( pev )
    return( pev[2] );
  return( m_pheader->serial_number );
}


//
int
E391Detector::eventNumber() const
{
  const int*  pev = (int*)pbank( "EVNT" );
  if ( pev )
    return( pev[3] );
  return( -1 );
}


//
void*
E391Detector::pbank( const std::string& bankname ) const
{
  if ( DEBUG )
    std::cout << "E391Detector::pbank()" << std::endl;
  
  DWORD*  pdata;
  bk_locate( m_pevent, (char*)bankname.c_str(), &pdata );
  return( pdata );
}


//
bool
E391Detector::exist( const std::string& bankname ) const
{
  if ( DEBUG )
    std::cout << "E391Detector::exist()" << std::endl;

  DWORD*  pdata;
  int  n = bk_locate( m_pevent, (char*)bankname.c_str(), &pdata );
  if ( n ){
    return( true );
  }
  return( false );
}

