// -*- C++ -*-
//
// MidasFileIO.cc
// FileIO class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Jul 10 18:40:57 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include "midas.h"
#include "MidasManager/MidasBank.h"


//////
#define DEBUG  0


//////
MidasBank::MidasBank( int bufsize )
  : m_bufsize( bufsize )
{
  if (DEBUG) std::cerr << "MidasBank::MidasBank" << std::endl;
  m_unalignedbuf = new char[ m_bufsize+16 ];
}


//////
MidasBank::MidasBank( const MidasBank& mb )
  : m_bufsize( mb.m_bufsize )
{
  if (DEBUG) std::cerr << "MidasBank::MidasBank copy" << std::endl;

  m_unalignedbuf = new char[ mb.m_bufsize+16 ];
  memcpy( (char*)ALIGN((PTYPE)m_unalignedbuf),
          (char*)ALIGN((PTYPE)mb.m_unalignedbuf),
          m_bufsize );
}


//////
MidasBank::~MidasBank()
{
  if (DEBUG) std::cerr << "MidasBank::~MidasBank" << std::endl;

  delete m_unalignedbuf;
}


//////
MidasBank&
MidasBank::operator=( const MidasBank& mb )
{
  if (DEBUG) std::cerr << "MidasBank::operator=" << std::endl;

  delete m_unalignedbuf;
  m_unalignedbuf = new char[ mb.m_bufsize+16 ];
  memcpy( (char*)ALIGN((PTYPE)m_unalignedbuf), 
          (char*)ALIGN((PTYPE)mb.m_unalignedbuf),
          m_bufsize );
  return( *this );
}


//////
bool
MidasBank::realloc( int size )
{
  if (DEBUG)
    std::cerr << "MidasBank::realloc :"
              << " size= " << size
              << " m_buf= " << m_bufsize
              << std::endl;

  if ( size > m_bufsize ){
    m_bufsize = size;
    delete  m_unalignedbuf;
    m_unalignedbuf = new char[ m_bufsize+16 ];
    if ( m_unalignedbuf )
      return( true );
  }
  return( false );
}


//////
void
MidasBank::init( int event_id,int mask, int serialnumber )
{
  if (DEBUG) std::cerr << "MidasBank::init" << std::endl;

  bm_compose_event( eventHeader(), event_id, mask, 0, serialnumber );
  bk_init32( eventHeader()+1 );
}


//////
int
MidasBank::getTable( char* name,size_t size, void* pdata )
{
  if (DEBUG) std::cerr << "MidasBank::getTable" << std::endl;

  // look for the bank
  int  rsize = bk_locate( eventHeader()+1, name, pdata );
  return( rsize / size );
}


//////
int
MidasBank::addTable( char* name,int TID_type, void* pdata, int n_elements )
{
  if (DEBUG) std::cerr << "MidasBank::addTable" << std::endl;

  // 
  char*  pbuf;
  int    bytes = TID_type * n_elements;
  bk_create( eventBuffer(), name, TID_type, &pbuf );
  BANK*  bh = (BANK*)pbuf;
  memcpy( pbuf, pdata, bytes );
  pbuf += bytes;
  int  ret = bk_close( eventBuffer(), pbuf );
  return( ret );
}

