// -*- C++ -*-
//
// Detector.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include "detparam/DetParam.h"
#include "detector/Detector.h"
#include "E391Detector/E391Detector.h"

//

//
Detector::Detector()
  : m_DB( 0 )
{
  //std::cout << "Detector::Detector()" << std::endl;

}


//
Detector::~Detector()
{
  //std::cout << "Detector::Detector()" << std::endl;

}


//
int
Detector::adc() const
{
  //std::cout << "Detector::adc()" << std::endl;
  if ( m_DB ){
    const E391Detector&  e391det = E391Detector::instance();
    return( e391det.adcCrate(m_DB->adcCrate).adc(m_DB->adcSlot)
            .data(m_DB->adcChannel) );
  }

  return( -1 );
}


//
int
Detector::tdc() const
{
  //std::cout << "Detector::tdc()" << std::endl;
  if ( m_DB ){
    const E391Detector&  e391det = E391Detector::instance();
    return( e391det.tdcCrate(m_DB->tdcCrate).tdc(m_DB->tdcSlot)
            .data(m_DB->tdcChannel) );
  }

  return( -1 );
}


//
double
Detector::charge() const
{
  //std::cout << "Detector::charge()" << std::endl;
  if ( m_DB ){
    const E391Detector&  e391det = E391Detector::instance();
    unsigned int  dt = adc();
    int  rawadc = dt & 0x0000ffff;
    int  range  = dt & 0xffff0000;
    double  ped = (range ? m_DB->pedestalH : m_DB->pedestalL);
    double  chg = (rawadc - ped) * m_DB->gain;
    return( chg );
  }

  return( 0 );
}


//
double
Detector::time() const
{
  //std::cout << "Detector::time()" << std::endl;
  if ( m_DB ){
    const E391Detector&  e391det = E391Detector::instance();
    unsigned int  dt = tdc();
    double  ped = 0;
    double  tim = (dt - ped) * m_DB->timeResol;
    return( tim );
  }

  return( 0 );
}

