// -*- C++ -*-
//
// DetParam.h
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#ifndef DETPARAM_H_INCLUDED
#define DETPARAM_H_INCLUDED

#include <iostream>
#include <list>
#include <string>

//
#define  MAX_NADCCRATE   2
#define  MAX_NADC       23
#define  MAX_NADCCH     96
#define  MAX_NTDCCRATE   2
#define  MAX_NTDC       23
#define  MAX_NTDCCH     64


// # det type subID ID ADCpa ADCsa TDCma TDCsa x y z HVcr HVsl HVch cable gain pedL pedH Tresol
typedef struct{
  std::string  name;
  std::string  type;
  std::string  cable;
  int          clusterID;
  int          ID;
  int          adcCrate;
  int          adcSlot;
  int          adcChannel;
  int          tdcCrate;
  int          tdcSlot;
  int          tdcChannel;
  double       x;
  double       y;
  double       z;
  double       hvCrate;
  double       hvSlot;
  double       hvChannel;
  double       gain;
  double       pedestalL;
  double       pedestalH;
  double       timeResol;
} E391_DETE_DB;


//
class DetParam{
public:
  // singleton
  static DetParam&  instance();
  //
  const std::list<E391_DETE_DB>&  paramList() const { return m_paramList; }
  const E391_DETE_DB*  db_adc( int crate, int slot, int ch ) const; // reverse index
  const E391_DETE_DB*  db_tdc( int crate, int slot, int ch ) const; // reverse index
  //
  bool  getDB( int runno );
  void  dumpDB() const;

protected:
  DetParam();
  ~DetParam();

private:
  // singleton
  static DetParam*  s_detparam;
  // data
  std::list<E391_DETE_DB>  m_paramList;
  E391_DETE_DB*  m_db_adc[MAX_NADCCRATE][MAX_NADC][MAX_NADCCH]; // reverse index
  E391_DETE_DB*  m_db_tdc[MAX_NTDCCRATE][MAX_NTDC][MAX_NTDCCH]; // reverse index
  // method
  void    makeReverseIndex();

};

//
#endif // DETPARAM_H_INCLUDED

