// -*- C++ -*-
//
// DetParam.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "detparam/DetParam.h"

#define DEBUG  0


// static members
DetParam*  DetParam::s_detparam = 0;

static const E391_DETE_DB  db_dummy = {};


// singleton
DetParam&
DetParam::instance()
{
  //std::cout << "DetParam::instance()" << std::endl;
  
  if ( s_detparam == 0 ){
    s_detparam = new DetParam;
    if ( s_detparam == 0 ){ // failed
      std::cout << "DetParam::instance() : cannot create DetParam object."
                << std::endl;
      perror( "new" );
      abort();
    }
  }

  return( *s_detparam );
}


//
DetParam::DetParam()
{
  if ( DEBUG )
    std::cout << "DetParam::DetParam()" << std::endl;
}

//
DetParam::~DetParam()
{
  if ( DEBUG )
    std::cout << "DetParam::DetParam()" << std::endl;
}

//
const E391_DETE_DB*
DetParam::db_adc( int crate, int slot, int ch ) const
{
  if ( DEBUG )
    std::cout << "DetParam::db_adc()" << std::endl;

  if ( 0 <= crate && crate < MAX_NADCCRATE && 
       0 <= slot  && slot  < MAX_NADC      &&
       0 <= ch    && ch    < MAX_NADCCH     ){
    return( m_db_adc[crate][slot][ch] );
  }
  return( NULL );
}

//
const E391_DETE_DB*
DetParam::db_tdc( int crate, int slot, int ch ) const
{
  if ( DEBUG )
    std::cout << "DetParam::db_tdc()" << std::endl;

  if ( 0 <= crate && crate < MAX_NTDCCRATE && 
       0 <= slot  && slot  < MAX_NTDC      &&
       0 <= ch    && ch    < MAX_NTDCCH     ){
    return( m_db_tdc[crate][slot][ch] );
  }
  return( NULL );
}

//
bool
DetParam::getDB( int runno )
{
  if ( DEBUG )
    std::cout << "DetParam::getDB()" << std::endl;

  // clear list
  m_paramList.clear();

  //
  char* s[2];
  s[0] = getenv( "MY_TOP_DIR" );
  s[1] = getenv( "E391_TOP_DIR" );

  for( int i=0;i<2;i++ ){
    char  ss[64];
    sprintf( ss,"run%05d.db",runno );
    static const std::string  dname = "/share/E391Detector/";
    std::string  fname = s[i] + dname + ss;
    std::ifstream  ifile( fname.c_str() );
    if ( ifile.good() ){
      std::cout << "opening DB : " << fname << std::endl;
      // readDB
      char  buf[1024];
      while( ifile.getline( buf,1024 ), !ifile.eof() ){
        char*  str  = buf;
        char*  next = str;
        // #name type clusID ID Acr Apa Asa Tcr Tma Tsa x y z HVcr HVsl HVch cable gain pedL pedH Tresol
        E391_DETE_DB  db = db_dummy;

#ifdef OS_SOLARIS
#define  strsep(x,y) strtok(*(x),(y))
#endif

        if ( str = strsep( &next," \t\0\n" ) ){
          if ( str[0] == '#' || str[0] == '!' || str[0] == ';' || 
               strlen(str) == 0 ){
            continue; // comments
          }
        }
        db.name = str;

#ifdef OS_SOLARIS
#undef   strsep
#define  strsep(x,y) strtok(NULL,(y))
#endif
        if ( str = strsep( &next," \t\0\n" ) ){
          db.type = str;
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.clusterID = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.ID = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.adcCrate = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.adcSlot = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.adcChannel = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.tdcCrate = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.tdcSlot = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.tdcChannel = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.x = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.y = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.z = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.hvCrate = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.hvSlot = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.hvChannel = (int)strtol( str,NULL,0 );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.cable = str;
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.gain = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.pedestalL = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.pedestalH = strtod( str,NULL );
        }
        if ( str = strsep( &next," \t\0\n" ) ){
          db.timeResol = strtod( str,NULL );
        }
        m_paramList.push_back( db );
      }
      // create reverse index for ADC,TDC
      makeReverseIndex();
      // end
      std::cout << m_paramList.size() << " records read." << std::endl;

      if ( DEBUG )
        dumpDB();

      return( true );
    }
  }
  
  std::cout << "cannot find DB for Run " << runno << std::endl;
  return( false );
}


//
void
DetParam::makeReverseIndex()
{
  if ( DEBUG )
    std::cout << "DetParam::makeReverseIndex()" << std::endl;

  // clear
  for( int i=0; i<MAX_NADCCRATE; i++ )
    for( int j=0; j<MAX_NADC; j++ )
      for( int k=0; k<MAX_NADCCH; k++ )
        m_db_adc[i][j][k] = NULL;
  for( int i=0; i<MAX_NTDCCRATE; i++ )
    for( int j=0; j<MAX_NTDC; j++ )
      for( int k=0; k<MAX_NTDCCH; k++ )
        m_db_tdc[i][j][k] = NULL;

  for( std::list<E391_DETE_DB>::iterator i=m_paramList.begin();
       i!= m_paramList.end(); i++ ){
    int  acr = i->adcCrate;
    int  apa = i->adcSlot;
    int  asa = i->adcChannel;
    if ( 0 <= acr && acr < MAX_NADCCRATE && 
         0 <= apa && apa < MAX_NADC      && 
         0 <= asa && asa <= MAX_NADCCH   ){
      m_db_adc[acr][apa][asa] = &(*i);
    }
    int  tcr = i->tdcCrate;
    int  tma = i->tdcSlot;
    int  tsa = i->tdcChannel;
    if ( 0 <= tcr && tcr < MAX_NTDCCRATE &&
         0 <= tma && tma < MAX_NTDC      &&
         0 <= tsa && tsa <= MAX_NTDCCH   ){
      m_db_tdc[tcr][tma][tsa] = &(*i);
    }
  }
}


//
void
DetParam::dumpDB() const
{
  if ( DEBUG )
    std::cout << "DetParam::dumpDB()" << std::endl;

  // #name type clusID ID Acr Apa Asa Tcr Tma Tsa x y z HVcr HVsl HVch cable gain pedL pedH
  for( std::list<E391_DETE_DB>::const_iterator i=m_paramList.begin();
       i!= m_paramList.end(); i++ ){
    std::cout << i->name << " " 
              << i->type << " " 
              << i->clusterID << " " 
              << i->ID << " " 
              << i->adcCrate   << " " 
              << i->adcSlot    << " " 
              << i->adcChannel << " " 
              << i->tdcCrate   << " " 
              << i->tdcSlot    << " " 
              << i->tdcChannel << " " 
              << i->x << " " 
              << i->y << " " 
              << i->z << " " 
              << i->hvCrate << " " 
              << i->hvSlot << " " 
              << i->hvChannel << " " 
              << i->cable << " " 
              << i->gain << " " 
              << i->pedestalL << " " 
              << i->pedestalH << " "
              << i->timeResol << " "
              << std::endl;
  }
  
}


