// -*- C++ -*-
//
// TdcEvent.cc
//
// Mon Jul 15 09:00:04 JST 2002
// Mitsuhiro YAMAGA
//
// $Id$
//
// $Log$
//

#include <iostream>
#include <iomanip>
#include "midas.h"
#include "unpack/TdcModule.h"
#include "unpack/TdcEvent.h"

#define DEBUG 0

static const EVENT_HEADER  zero_eh = {0,0,0,0,0};


//
TdcEvent::TdcEvent()
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( -1 ),
    m_flag( 0 ),
    m_valid( false )
{
  //
}


//
TdcEvent::TdcEvent( int crate, const EVENT_HEADER* pheader, const DWORD* pdata,
                    int banksize, int nth_event )
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( crate ),
    m_flag( 0 ),
    m_valid( false )
{
  //
  unpack( m_crate, pheader, pdata, banksize, nth_event );
}


//
TdcEvent::~TdcEvent()
{

}

//
const TdcModule&
TdcEvent::tdc( int ma ) const
{
  for( std::list<TdcModule>::const_iterator i = m_tdcList.begin();
       i != m_tdcList.end(); i++ ){
    if ( i->ma() == ma )
      return( *i );
  }
  static const TdcModule  dummy;
  return( dummy );
}

//
int
TdcEvent::unpack( int crate, const EVENT_HEADER* pheader, const DWORD* pdata,
                  int banksize, int nth_event )
{
  // remove the previous data
  m_tdcList.clear();
  m_serial = -1;

  // crate
  m_crate = crate;

  // event header
  if ( pheader )
    m_eventHeader = *pheader;

  // start
  m_valid = false;
  for( int i=0;i<banksize;i++ ){
#if DEBUG
    std::cout << std::setw(8) << std::setfill('0') << std::hex 
              << pdata[i] << std::dec << std::endl;
#endif
    if ( pdata[i] == 0xffffffff ){
#if DEBUG
      std::cout << "Endmark found." << std::endl;
#endif
      break;
    }
    // skip
    int  serial = (pdata[i] >> 16) & 0xffff;
    int  nhit   = pdata[i] & 0xffff;
#if DEBUG
    std::cout << "Serial = " << serial << " nhit=" << nhit << std::endl;
#endif
    if ( nth_event > 0 && serial != nth_event ){
      i += nhit;
      continue;
    }
    // Nth event
    m_valid = true;
    m_serial = serial;
    TdcModule*  tm = NULL;
    int  oldma = -1;
    for( int n=0;n<nhit;n++ ){
      int  data = pdata[i+1+n];
      if ( (data & 0xff000000) == 0xff000000 ){
        m_flag = data;
        break;
      }
      // SMP data structure
      int  ma = (data >> 27) & 0x1f;
      int  ch = (data >> 16) & 0x7ff;
      int  dt = data & 0xffff;
      if ( ma != oldma ){
        if ( tm != NULL ){
#if DEBUG
          tm->show();
#endif
          m_tdcList.push_back( *tm );
          delete tm;
          tm = NULL;
        }
        tm = new TdcModule( crate,ma );
        oldma = ma;
      }
      tm->setData( ch,dt );
      tm->hitList().push_back( data );
    }
    if ( tm != NULL ){
#if DEBUG
      tm->show();
#endif
      m_tdcList.push_back( *tm );
      delete tm;
      tm = NULL;
    }
    i += nhit;
  }
#if DEBUG
  std::cout << "Event end." << std::endl;
#endif

  if ( m_valid )
    return( m_tdcList.size() ); // nth_event found.
  return( -1 ); // No such serial # in the spill.
}


//
int
TdcEvent::nhit() const
{
  int  n = 0;
  for( std::list<TdcModule>::const_iterator m = m_tdcList.begin();
       m != m_tdcList.end(); m++ ){
    n += m->hitList().size();
  }
  return( n );
}


//
int
TdcEvent::smp( const DWORD* pdata, int banksize )
{
  for( int i=0;i<banksize;i++ ){
    m_smp.push_back( *pdata++ );
  }
  return( m_smp.size() );
}


//
int
TdcEvent::smp( const std::list<unsigned int>& s )
{
  m_smp = s;
  return( m_smp.size() );
}

